	OPTIONS/EXTEND
	INTEGER*4 FUNCTION CONNECT_NODE(NRB)
	IMPLICIT NONE
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&CONNECT_NODE\&
C .nf
C .x CONNECT_NODE>Defined
C Source:CONNECT_NODE.MOD
C Designer           :EARL LAKIA
C Author             :EARL LAKIA
C             Inland Steel
C             Process Automation Department
C             3210 Watling St.  MS 2-465
C             East Chicago, IN  46312
C  
C Date of last update: 28-jun-1988
C Revision level     :1.7
C  
C .C ;Formal Parameter List
C Receives:
C  
C    NRB                        STRUCTURE                                         
C                               ROUTING DATABASE  NODE ROUTING BLOCK
C  
C Returns:
C  
C    CONNECT_NODE              INTEGER*4                                         
C                               STATUS RETURN FOR THIS SUBROUTINE                 
C  
C Accesses common(s):
C  
C .X MA_ROUTER_COMMON>Common
C    MA_ROUTER_COMMON           MA_ROUTER COMMON PARAMETERS SHARED THROUGHTOUT    
C  
C .X ROUTE_DBS>Common
C    ROUTE_DBS                  ROUTING DATABASE, HOWEVER, ITS ADDRESSES IS       
C                               PASSED AS A PARAMETER                             
C  
C Accesses file(s):
C  
C Other modules referenced:
C  
C .X OPR_FAO_MSG>Referenced
C    OPR_FAO_MSG                USED TO WRITE TO CONSOLE                          
C                                (found in library:INLAND)
C  
C 
C .X DISCONNECT_NODE>Referenced
C    DISCONNECT_NODE            IF CONNECT FAILS, CALL TO DISCONNECT
C                               AND START RETRY TIMERS
C 
C .X NON_BLANK_LEN>Referenced
c                               STRING LENGTH WITHOUT BLANKS
C                                (found in library:INLAND)
C .SK
C .fill
C .SK
C Description:
C .sk    
C 	This subroutine is responsible for doing the initial 
C connection attempts to  a remote node.  To do this, the
C subroutine must build NCBs and request connection.  The
C AST will handle the actual completion of the connection.  The
C channel number will also be stored in the routing database.
c 	This subroutine is also called to connect (accept 
c a connection) for remote nodes connecting to us.  In this 
c case, the NCB is not built by this routine as it was already
c set up by BUILD_REMOTE_CONNECT_NCB.
C 
C CALLING SEQUENCE
C    SYS_STATUS=CONNECT_NODE(NRB)
C  
C end.doc ****************************** end.doc  
C
C MODIFICATION HISTORY
c 	1.7 28-JUN-1988  E. LAKIA
C	Allow router to connect to nodes with names less than 6 chracters.
C	1.3 24-FEB-1987  E. LAKIA
C	Added support for logical primary node
c	1.4 27-feb-1987  E. lakia
c	fixed bug from 1.3

	INCLUDE '($SSDEF)'
	INCLUDE '($IODEF)'
	INCLUDE 'MA_ROUTER_COMMON.INC/LIST'
	INCLUDE 'MA_ROUTER_MSG.INC/NOLIST'
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(NRBDEF)/LIST'
C
	INTEGER*4 SYS_STATUS	! SYSTEM SERVICE STATUS
	INTEGER*4 SYS$QIO	! QIO SERVICE
	INTEGER*4 SYS$ASSIGN	! ASSIGN
	INTEGER*4 LIB$ASN_WTH_MBX  ! ASSIGN TO NETWORK WITH A MBX
	INTEGER*4 STR$POSITION
	INTEGER*4 SYS$GETMSG	! GET SYSTEM MESSAGE
	INTEGER*4 SYS$BINTIM	! TIME COVERSION
	INTEGER*4 SYS$DASSGN	! DEASSIGN CHANNEL
	INTEGER*4 SYS$EXIT	! SYSTEM EXIT
	integer*4 sys$setimr	! set timer
	INTEGER*4 SYS$CREPRC	! create process
	INTEGER*4 OPR_FAO_MSG	! OPERATOR MESSAGES
	INTEGER*4 NON_BLANK_LEN	! STRING LENGTH WITHOUT BLANKS
	INTEGER*4 RETRY_TIMER(2) ! RETRY TIMER
	INTEGER*4 IS		! Number characters in node names
C
	RECORD/NRBDEF/NRB	! NRB

C
C If we are attempting to connect a logical node name and we don't
c know where the node is really, then don't try to attempt the
c connection.
c
	IF(((NRB.NRB$L_FLAGS .AND. NRB$M_PRILOG).NE. 0)	! PRIMARY LOGICAL?
	1	.AND. 				! IS LOGICAL NODE TRANSLATED?
	1   (NRB.NRB$C_SECNODE(1:1) .EQ. CHAR(0) ))THEN	! NO....
	   SYS_STATUS=SYS$CREPRC(,		! PID
	1			IMAGE_NAME,	! REQUEST FIND SERVICE
	1			SYSINPUT,	! PROCESS SYS$INPUT
	1			SYSOUTPUT,	! PROCESS SYS$OUTPUT
	1			SYSERROR,	! PROCESS SYS$ERROR
	1			,,	! PRIV, QUOTA
	1			PROC_NAME,
	1			,,,)	! BASE PRI, UIC, MBX UNIT, STATUS
	   IF(.NOT. SYS_STATUS .AND.	! BAD STATUS( DUPLICATE NAME OK TOO)?
	1	SYS_STATUS .NE. SS$_DUPLNAM)THEN
		    MSGDESC(1)=512
	 	    SYS_STATUS=SYS$GETMSG(%VAL(SYS_STATUS),
	1	    MSGDESC(1),MSGDESC,,)
	  	    SYS_STATUS=OPR_FAO_MSG(RTR_CREPRC,MSGDESC)
	   ENDIF
C
C	   START A SHORT TIMER SO WILL CONNECT LATER AFTER FIND 
C	   SERVICE COMPLETES.
C
	   SYS_STATUS = SYS$BINTIM('0 :01:00',
	1			RETRY_TIMER)
	   IF(.NOT. SYS_STATUS)THEN
		  MSGDESC(1)=512
		  SYS_STATUS=SYS$GETMSG(%VAL(SYS_STATUS),
	1	     MSGDESC(1),MSGDESC,,)
	   	  SYS_STATUS=OPR_FAO_MSG(RTR_RTYTIM,MSGDESC,NRB.NRB$Q_NCBDES)
		  SYS_STATUS=SYS$EXIT(%VAL(SYS_STATUS))
	    ENDIF
C
C	    ACTUALLY START THE TIMER
C
	    SYS_STATUS = SYS$SETIMR(,	! EVENT FLAG
	1		%REF(RETRY_TIMER),	! TIMER VALUE
	1		%VAL(CON_TIMER_AST_ADD), ! AST ADDRESS
	1		NRB)		 ! REQUEST ID (= NRB ADDRESS)
	    IF(.NOT. SYS_STATUS)THEN
		  MSGDESC(1)=512
		  SYS_STATUS=SYS$GETMSG(%VAL(SYS_STATUS),
	1	     MSGDESC(1),MSGDESC,,)
	   	  SYS_STATUS=OPR_FAO_MSG(RTR_RTYTIM,MSGDESC,NRB.NRB$Q_NCBDES)
		  SYS_STATUS=SYS$EXIT(%VAL(SYS_STATUS))  ! FATAL BUG ERROR
	    ENDIF
	    NRB.NRB$L_FLAGS=NRB.NRB$L_FLAGS .OR.
	1	 NRB$M_CONTIMER	! FLAG AS HAVING A TIMER ACTIVE

	    RETURN
	ENDIF  ! end of a logical type node name test 

C
C SEE IF REMOTE HAS REQUESTED A CONNECT ALREADY OR IF SOMEONE HAS 
C LOCALLY REQUESTED A DISCONNECT
C
	IF((NRB.NRB$L_FLAGS .AND. 
	1	(NRB$M_DRQ .OR. NRB$M_ATM .OR. NRB$M_SHUTDOWN))
	1	.NE. 0)THEN	! WELL HAS SOMEONE?
		CONNECT_NODE=SS$_NORMAL	! YES, WELL THAT OK STILL
		RETURN			! JUST A LOT LESS WORK FOR US.
	ENDIF
C
	NRB.NRB$L_FLAGS=NRB.NRB$L_FLAGS .OR. NRB$M_ATM	! FLAG AS CONNECT
							! IN PROGRESS
C
C BUILD A DESCRIPTOR FOR THE NCB UNLESS IT WAS REMOTLY REQUESTED
C
	NRB.NRB$C_CURNODE='        ' ! Init it to all blanks
	IF((NRB.NRB$L_FLAGS .AND. NRB$M_RNRQC) .NE. 0)THEN  ! REMOTE REQ?
c
c	   REMOTE REQUESTING CONNECTION (copy node name from the
c	   NCB)
C
	   IS=STR$POSITION(NRB.NRB$C_REMNCB(1:6),':') ! How long is the node name
	   IF(IS .LE. 0 .OR. IS .GT. 6)THEN
		IS=6
	   ELSE
		IS=IS-1
	   ENDIF
	   NRB.NRB$C_CURNODE(7-IS:8)=NRB.NRB$C_REMNCB(1:IS) // '::'  ! copy node name
	ELSE		
C
C	  HOST REQUESTING CONNECTION (fill in correct node name with ::)
c
	  IF((NRB.NRB$L_FLAGS .AND. NRB$M_SEC) .NE. 0)THEN ! PRIMARY OR SECONDARY
c
c		SECONDARY
C
	 	IS=STR$POSITION(NRB.NRB$C_SECNODE,' ') ! length of node name
		IF(IS .LE. 0 .OR. IS .GT. 6) THEN
	  	  IS=6
		ELSE
		  IS=IS-1
		ENDIF
		NRB.NRB$C_CURNODE(7-IS:8)= NRB.NRB$C_SECNODE(1:IS) // '::' ! SECONDARY NODE
	  ELSE
C
C		PRIMARY
C
	 	IS=STR$POSITION(NRB.NRB$C_PRINODE,' ') ! length of node name
		IF(IS .LE. 0 .OR. IS .GT. 6) THEN
	  	  IS=6
		ELSE
		  IS=IS-1
		ENDIF
		NRB.NRB$C_CURNODE(7-IS:8)= NRB.NRB$C_PRINODE(1:IS) // '::' ! PRIMARY NODE
	  ENDIF ! end primary/secondary node test
C
C 		Build an NCB (is= size of node name)  The node name is right
c 		justified.  Therefore, we add an offset to the character descritor
c 		to start at the beginning of the node name, and adjust the size
c 		of the byte count for the node name and access control.
C
C
		NRB.NRB$Q_NCBDES(1)= IS + 2 + 		! Byte count
	1		NON_BLANK_LEN(NRB.NRB$C_ACC,NRB.NRB$C_ACC)
		NRB.NRB$Q_NCBDES(2)= %LOC(NRB.NRB$C_CURNODE) + (6-IS)
	ENDIF  ! End of remote versus host connect attempt

C
C ASSIGN A CHANNEL TO THE NETWORK
C
1000	CONTINUE
d	CONNECT_NODE=OPR_FAO_MSG(RTR_CONTRY,NRB.NRB$Q_NCBDES)	! DEBUG
	CONNECT_NODE=LIB$ASN_WTH_MBX('_NET:',
	1	NET_MBX_SIZE,	! SIZE OF MAIL BOX
	1	NET_MBX_SIZE*3,	! BUFFER QUOTA, ALLOW THREE NET MESSAGES
	1	NRB.NRB$L_CHAN,			! CHANNEL ASSIGNED TO ACP
	1	NRB.NRB$L_NETMBX)		! CHANNEL ASSIGNED TO MBX,
C						! WILL GET DISCONNECTS, ETC.
	IF(.NOT. CONNECT_NODE)THEN	! 
		MSGDESC(1)=512
		SYS_STATUS=SYS$GETMSG(%VAL(CONNECT_NODE),MSGDESC(1),
	1	MSGDESC,,)
	   	SYS_STATUS=OPR_FAO_MSG(RTR_NETASN,MSGDESC,NRB.NRB$Q_NCBDES)
		SYS_STATUS=SYS$EXIT(%VAL(CONNECT_NODE))
		RETURN
	ENDIF
C
C PLACE READ ON THE NETWORK MAILBOX TO GET THE DISCONNECT MESSAGES
C AND THE CONNECT CONFIRMATION MESSAGES FROM THE REMOTE
C
	CONNECT_NODE=SYS$QIO(
	1	,			! EVENT FLAG
	1	%VAL(NRB.NRB$L_NETMBX),	! CHANNEL ASSIGNED TO NETWORK MBX
	1	%VAL(IO$_READVBLK ), 	! READ FUNCTION
	1	NRB.NRB$Q_MBXSTS,	! I/O STATUS BLOCK	
	1	%VAL(NET_AST_ADD),	! AST ADDRESS
	1	NRB,			! AST PARAMETER= NRB
	1	NRB.NRB$L_NETMSG,	 ! BUFFER TO READ LETTER INTO
	1	%VAL(NET_MBX_SIZE),,,,)		 ! SIZE OF LETTER BUFFER
C
	IF(.NOT. CONNECT_NODE)THEN
		MSGDESC(1)=512
		SYS_STATUS=SYS$GETMSG(%VAL(CONNECT_NODE),MSGDESC(1),
	1	MSGDESC,,)
		SYS_STATUS=OPR_FAO_MSG(RTR_NETMBX,MSGDESC)
		RETURN
	ENDIF
C
C  DO THE QIO TO ACTUALLY REQUEST THE CONNECTION
C
	CONNECT_NODE=SYS$QIO(  , 	! EVENT FLAG
	1	%VAL(NRB.NRB$L_CHAN),	! CHANNEL ASSIGNED TO NETACP
	1	%VAL(IO$_ACCESS),	! FUNCTION CODE
	1	NRB.NRB$Q_NETSTS,	! IO STATUS
	1	%VAL(CONNECT_NODE_AST_ADD),	! AST ROUTINES
	1	NRB,			! AST PARAMETER= NRB
	1	,			! P1, NOT USED
	1	NRB.NRB$Q_NCBDES, 	! NCB DESCRIPTOR
	1	,,,)

C
C CHECK STATUS OF THE CONNECT REQUEST
C
	IF(.NOT. CONNECT_NODE)THEN	! 
	  IF( CONNECT_NODE  .NE.	! SAME STATUS AS LAST TIME
	1	NRB.NRB$Q_OLDSTS.COND)THEN	! NO, THEN LOG THE NEW ONE
		MSGDESC(1)=512
		SYS_STATUS=SYS$GETMSG(%VAL(CONNECT_NODE),MSGDESC(1),
	1	MSGDESC,,)
	   	SYS_STATUS=OPR_FAO_MSG(RTR_CONFAL,MSGDESC,NRB.NRB$Q_NCBDES)
	  ENDIF
C
	  NRB.NRB$Q_OLDSTS.COND=CONNECT_NODE ! COPY OLD=NEW STATUS
	  CALL DISCONNECT_NODE(NRB)	! CALL TO DISCONNECT THE NODE
	  RETURN
C
	ENDIF
	RETURN	
	END
