	OPTIONS/EXTEND
	INTEGER*4 FUNCTION READ_NET_AST(BUF,R0,R1,PC,PSL)
	IMPLICIT NONE
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&READ_NET_AST\&
C .nf
C .x READ_NET_AST>Defined
C Source:READ_NET_AST.FOR
C Designer           :EARL LAKIA
C Author             :EARL LAKIA
C             Inland Steel
C             Process Automation Department
C             3210 Watling St.  MS 2-465
C             East Chicago, IN  46312
C  
C Date of last update: 18-FEB-1987
C Revision level     :2.4
C  
C .C ;Formal Parameter List
C Receives:
C  
C    BUF                        STRUCTURE/BUFDEF/                                 
C                               I/O STATUS, NRB LINK, AND DATA BUFFER STRUCTURE   
C  
C    R0,R1,PC,PSL               REGISTERS                                         
C                               SAVED REGISTERS FROM AST                          
C  
C Returns:
C  
C Accesses common(s):
C  
C 
C .X DISCONNECT_NODE
C    DISCONNECT_NODE            DISCONNECT A REMOTE LOGICAL LINK
C 
C .X EVENT_Q>Common
C    EVENT_Q                    QUEUE NETWORK READ COMPLETE MESSAGE AND           
C                               POSSIBLE ACKNOWLEGE                               
C  
C .X MA_ROUTER_COMMON>Common
C    MA_ROUTER_COMMON           MA ROUTER COMMON                                  
C 
C Accesses file(s):
C  
C Other modules referenced:
C  
C .X OPR_FAO_MSG>Referenced
C    OPR_FAO_MSG                LOG MESSAGES                                      
C                                (found in library:INLAND)
C  
C .X ALC_RESP_BUF>Referenced
C    ALC_RESP_BUF               Used to allocate a response buffer
c  
C .X ACK_LAST_WRITE>Referenced
c    ACK_LAST_WRITE             Acknowlege message to queue
c                               
C 
C .X RTR_WRITE_Q>Referenced 
C    RTR_WRITE_Q                Write a message to the Queue via router 
C  
C .X NET_READ>Referenced
c    NET_READ                   DO A NETWORK READ
C  
C .X ACK_READ>Referenced
C    ACK_READ                   Acknowledge a read message that was 
C                               successfully transmitted and queued
C                               on the remote node.
C 
C .X CHECK_LNK_SEQ>Referenced
c    CHECK_LNK_SEQ              Check link sequence number 
C 
C .X WAKE_RECEIVER>Referenced
C    WAKE_RECEIVER              Wake up a receiver process connected
C                               to a particular message id.
c 
C .X BUILD_RESPONSE>Referenced
c    BUILD_RESPONSE             Used to build and send the response
c                               to the remote node.
C 
C .X MARK_FULL>Referenced      
c    MARK_FULL                  Marks a message id listhead with the status
c                               that the remote node's queue is full
c
c .X MARK_NOROUTE>Referenced
c    MARK_NOROUTE              Marks a message id listhead with the status
c                              that it is not routeable, (remote node does
c                              not known of the message id)
C .SK
C .fill
C .SK
C Description:
C .sk    
C 	This subroutine is an AST subroutine called when
C a NETWORK read QIO completes.  The "BUF" parameter was
C the AST parameter in the QIO and points the a structure
C containing the I/O Status block, a link to the NRB, and
C the actual data.
C 	The message can be: 
C .list
C .le;A message that needs to be queued, in which case call the
C subroutine RTR_WRITE_Q and build an acknowlege message to
C send back to the sending node.  If there is a receiving process
C connected for this message ID, and the count goes from zero
C to one, then call WAKE_RECEIVER subroutine.
C .le;A response to a message that was written earlier by us
C in which case call the subroutine ACK_READ to acknowlege the
C read and have the message deleted from the queue.
C .els
c 	The proceedure that should be followed is:
c .list
C .le;If the I/O status was bad, call disconnect the node
C by deassigning the channel.  Also verify that the link
c sequence number received is one greater than the one 
c stored in the NRB.  If not, log the error.  Set the 
c link sequence number to the value in the message.
c .le;Attempt to acquire a response buffer. If no buffer
c can be acquired, then set up a marktime with this routine
c the AST for that marktime with the same parameters that
c this AST was called with from the QIO (when the marktime
c occurrs, we will think that the Read QIO just completed again)
c and return from this AST.
c .le;If this is a message to be queued,
c Add the message to the Queue, depending on the error status
c from the RTR_WRITE_Q subroutine, send the approiate status back 
c to the sending node.  The AST subroutine to service 
c this response QIO is  NET_READ_RESP_AST (it will deallocate 
c the response buffer).
c .le;If the message is an acknowlege of a write, then
c acknowlege the read from the queue (ACK_READ) if the 
c message is a posstive response, otherwise, back up the
c RNA pointer if the remote node message id queue is full,
c and mark the status in the message id list head.
c if the message is a response showing that the remote node
c does not know of the message id, then mark the message id
c as not routeable.
c .le;Queue another Read I/O for this node.
c .els
C 
C end.doc ****************************** end.doc  

	INTEGER*4 R1,R0,PSL,PC	! AST PARAMETERS THAT WE DON'T USE IN FORTRAN
	INCLUDE '($SSDEF)'
	INCLUDE '($IODEF)'
	INCLUDE '($MSGDEF)/NOLIST'	! MESSAGE DEFINITIONS
	INCLUDE 'MA_ROUTER_COMMON.INC/NOLIST'
	INCLUDE 'MA_ROUTER_MSG.INC/NOLIST'
	INCLUDE 'EVENT_Q.INC/NOLIST'
	RECORD/EVENT_ENTRY/EVENT
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(HDRDEF)/LIST'
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(MGHDEF)/LIST'
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(BUFDEF)/LIST'
	INCLUDE 'MA_Q:[DEF]QUEUE.TLB(NRBDEF)/LIST'
	RECORD/BUFDEF/BUF
C
	INTEGER*4 SYS_STATUS	! SYSTEM SERVICE STATUS
	INTEGER*4 SYS_STATUS2
	INTEGER*4 SYS$GETMSG	! GET SYSTEM MESSAGE
	INTEGER*4 SYS$DASSGN	! DEASSIGN CHANNEL
	INTEGER*4 SYS$QIO	! QIO FOR NETWORK MAILBOXES
	INTEGER*4 SYS$EXIT	! EXIT
	INTEGER*4 OPR_FAO_MSG	! OPERATOR MESSAGES
	INTEGER*4 PUSH_EVENT_QUE ! PLACE EVENT IN OUR QUEUE
	INTEGER*4 DISCONNECT_NODE ! DISCONNECT A LOGICAL LINK
	INTEGER*4 RTR_WRITE_Q	! WRITE TO QUEUE
	INTEGER*4 ACK_READ	! ACKNOWLEGE A READ
	INTEGER*4 WAKE_RECEIVER ! SEND A WAKE UP MESSAGE TO A CONNECTED PROCESS
	INTEGER*4 SYS$BINTIM	! ASCII TO BINARY TIME CONVERSION
	INTEGER*4 RESP_ADDR	! RESPONSE BUFFER ADDRESS
	INTEGER*4 ALC_RESP_BUF	! SUBROUTINE TO ALLOCATE RESPONSE BUFFER
	integer*4 sys$setimr	! set timer
	INTEGER*4 QM_DESCP(2)	! MESSAGE QUEUE DESCRIPTOR
C
C
	INTEGER*2 RETRY_TIMER(4)  ! QUAD WORD DELTA TIME FOR RETRY

D	SYS_STATUS=OPR_FAO_MSG(RTR_NTRDAST,
D	1	%VAL(BUF.BUF$Q_STS.TRANS))	! TELL SOMEONE WE RAN
C
	IF (.NOT. BUF.BUF$Q_STS.COND)THEN	! QIO ERROR?
	   MSGDESC(1)=512
	   SYS_STATUS=SYS$GETMSG(%VAL(BUF.BUF$Q_STS.COND),
	1	MSGDESC(1),MSGDESC,,)
	   SYS_STATUS=OPR_FAO_MSG(RTR_NETRDERR,MSGDESC)
C
	   IF(BUF.BUF$Q_STS.COND .EQ. SS$_INSFMEM .OR.	! FATAL ERROR?
	1	BUF.BUF$Q_STS.COND .EQ. SS$_PROTOCOL)THEN
			 GO TO 1000 			! NO,TRY AGAIN
	   ELSE
		CALL DISCONNECT_NODE(%VAL(BUF.BUF$L_NRB)) ! YES, DISCONNECT
		RETURN
	   ENDIF
C
C 	GOOD QIO, FIND OUT IF AN ACKNOWLEGE OR A NEW MESSAGE
C
	ELSE
D	  SYS_STATUS=OPR_FAO_MSG(RTR_NTRDMSG,	! FOR SOME DEBUG, 
D	1	BUF.BUF$S_MSG.MGH$C_MID,	! LOG THAT OUR MESSAGE HAS
D	1	BUF.BUF$S_MSG.HDR.HDR$C_SNM,	! ARRIVED
D	1	BUF.BUF$S_MSG.HDR.HDR$C_DNM,
D	1	%VAL(BUF.BUF$S_MSG.MGH$W_LSQ),
D	1	%VAL(BUF.BUF$S_MSG.HDR.HDR$W_TYPE),
D	1	%VAL(BUF.BUF$S_MSG.MGH$L_IDX),
D	1	%VAL(BUF.BUF$S_MSG.HDR.HDR$W_LEN),
D	1	%VAL(BUF.BUF$S_MSG.HDR.HDR$W_QSEQ))
C
	  CALL CHK_LNK_SEQ(BUF,%VAL(BUF.BUF$L_NRB))  ! HAVE LINK SEQ. CHECKE
C
	  IF(BUF.BUF$S_MSG.HDR.HDR$W_TYPE .LT. 0)THEN ! ROUTER MESSAGE?
C
C		ROUTER MESSAGE, CAN BE:
C		    1) ACKNOWLEGE 
C		    2) REMOTE MESSAGE ID QUEUE FULL 
C		    3) REMOTE NODE DOES NOT KNOW OF THIS ID
C

C
C		ACKNOWLEGE OF LAST WRITTEN MESSAGE
C		FIND THE INDEX FROM MESSAGE  AND THEN
C		ACKNOWLEGE THE MESSAGE TO THE QUEUE SYSTEM SERVICE
C		AND HANG ANOTHER READ.
C
		IF(BUF.BUF$S_MSG.HDR.HDR$W_TYPE .EQ. -1) THEN  ! ACKNOWLEGE
		CALL ACK_LAST_WRITE(%VAL(BUF.BUF$L_NRB), ! HAVE IT DONE BY SUBR.
	1		BUF,				! SO CAN PASS NRB
	1		%VAL(BUF.BUF$L_NRB+NRB$L_SIZ))	! AND NRBID LIST.
		GO TO 1000		! GO POST ANOTHER READ
C
C 		REMOTE MESSAGE ID FULL, IF SO PLACE MESSAGE BACK ON TO THE
C		QUEUE AND MARK MESSAGE ID LIST HEAD.
C
		ELSE IF(BUF.BUF$S_MSG.HDR.HDR$W_TYPE .EQ. -2)THEN ! FULL?
		     CALL PUT_BACK(BUF.BUF$S_MSG.MGH$C_MID, ! MESSAGE ID
	1		BUF.BUF$S_MSG.MGH$L_IDX,	! INDEX INTO NRBIDS
	1		%VAL(BUF.BUF$L_NRB+NRB$L_SIZ),	! NRBID LIST
	1		%VAL(BUF.BUF$L_NRB))		! NRB
C
		     CALL MARK_FULL(%VAL(BUF.BUF$L_NRB),
	1			BUF,Q_BLOCK)	! MARK IT AS FULL
C
		     SYS_STATUS=PUSH_EVENT_QUE(FULL_REMOTE) ! TELL MAINLINE TO
							    ! START TIMER TO 
							    ! TRY AGAIN LATER.
C
C		REMOTE NODE DOSEN'T KNOW OF THIS MESSAGE ID, WELL
C		WE WILL JUST CONTINUE TO QUEUE UNTIL SOMEDAY HE DOES.
C		THEREFORE, JUST MARK MESSAGE ID QUEUE SO WON'T TRY TO SEND
C		ANYMORE AND PLACE THE RNA MESSAGE BACK ON THE QUEUE.
C
		ELSE IF(BUF.BUF$S_MSG.HDR.HDR$W_TYPE .EQ. -3)THEN  ! REMOTE NODE
					! DOSEN'T KNOW OF THIS MESSAGE ID.
C
C		     PUT MESSAGE BACK ONTO THE FRONT OF THE QUEUE
C
		     CALL PUT_BACK(BUF.BUF$S_MSG.MGH$C_MID, ! MESSAGE ID
	1		BUF.BUF$S_MSG.MGH$L_IDX,	! INDEX INTO NRBIDS
	1		%VAL(BUF.BUF$L_NRB+NRB$L_SIZ),	! NRBID LIST
	1		%VAL(BUF.BUF$L_NRB))		! NRB
C
		     CALL MARK_NOROUTE(%VAL(BUF.BUF$L_NRB),
	1			BUF,Q_BLOCK)	! MARK IT AS NOT ROUTEABLE
		     SYS_STATUS=PUSH_EVENT_QUE(FULL_REMOTE) ! TELL MAINLINE TO
							    ! START TIMER TO 
							    ! TRY AGAIN LATER.
C
		ELSE IF(BUF.BUF$S_MSG.HDR.HDR$W_TYPE .EQ. -4)THEN  ! REMOTE NODE
C					! MESSAGE ID QUEUE NO LONGER FULL
		ELSE 
			GO TO 1000	! IGNORE MESSAGE THEN
		ENDIF

C
C	   A NEW MESSAGE TO BE QUEUED	
C
	   ELSE
		SYS_STATUS=ALC_RESP_BUF(  ! GET A RESPONSE BUFFER
	1	    %VAL(BUF.BUF$L_NRB),
	1	    %VAL(RESP_INUSE),%VAL(RESP_ADD_LST),  
	1	    RESP_ADDR)
C
		IF (.NOT. SYS_STATUS)THEN 	! COULDN'T GET ONE
C
C			NO RESPONSE BUFFER AVAILABLE, MARK TIME FOR
C			10 SECONDS AND COME BACK LATER AS IF QIO
C			HAD JUST HAPPENED.
C
			SYS_STATUS = SYS$BINTIM('0 :00:10',
	1			RETRY_TIMER)
			IF(.NOT. SYS_STATUS)THEN   ! ERROR? SAY WHAT
			  MSGDESC(1)=512
	   		  SYS_STATUS=SYS$GETMSG(SYS_STATUS,
	1			MSGDESC(1),MSGDESC,,)
	   		  SYS_STATUS=OPR_FAO_MSG(RTR_TIMFAL,MSGDESC)
			ENDIF
C
C			SET UP TIMER TO MAKE TIMEOUT LOOK LIKE I/O COMPLETE
C
			SYS_STATUS = SYS$SETIMR(,	! DON'T CARE ON EFN
	1			%REF(RETRY_TIMER),	! TIMER VALUE
	1			%VAL(READ_NET_AST_ADD),	! AST= US
	1			BUF)			! SAME BUFFER
			IF(.NOT. SYS_STATUS)THEN
			  MSGDESC(1)=512
	   		  SYS_STATUS=SYS$GETMSG(SYS_STATUS,
	1			MSGDESC(1),MSGDESC,,)
	   		  SYS_STATUS=OPR_FAO_MSG(RTR_TIMFAL,MSGDESC)
			ENDIF
		        RETURN  ! DON'T ISSUE ANOTHER READ YET
		ENDIF  ! TO HAVING A RESPONSE BUFFER

C
C 		GOT A RESPONSE BUFFER ALLOCATED, WE CAN NOW
C		STORE THE MESSAGE IN THE QUEUE (OR IF CAN'T)
C		AND ISSUE THE APPROIATE RESPONSE.
C
D		SYS_STATUS=OPR_FAO_MSG(RTR_QWRITEAT,
D	1	    BUF.BUF$S_MSG.MGH$C_MID,
D	1	    %VAL(BUF.BUF$S_MSG.HDR.HDR$W_LEN),
D	1	    %VAL(BUF.BUF$S_MSG.HDR.HDR$W_TYPE))
C
C
		QM_DESCP(1)=BUF.BUF$S_MSG.HDR.HDR$W_LEN+HDR$K_SIZ ! DESCRIPTOR
		QM_DESCP(2)=%LOC(BUF.BUF$S_MSG.HDR)  ! START OF BUF
C
		SYS_STATUS=RTR_WRITE_Q(	! WRITE IT TO THE QUEUE
	1		Q_BLOCK,		! Q BLOCK FOR QUEUE
	1		BUF.BUF$S_MSG.MGH$C_MID, ! MESSAGE ID
	1		QM_DESCP,		  ! BUFFER
	1		BUF.BUF$S_MSG.HDR.HDR$W_LEN,  ! LENGTH
	1		BUF.BUF$S_MSG.HDR.HDR$W_TYPE)	! TYPE OF MESSAGE
C
C		STATUS CAN BE: MESSAGE ID NOT KNOWN, MESSAGE PLACED
C			SUCCESSFULLY IN QUEUE OR THE QUEUE IF FULL.
C			ANYWAY MUST BE POLITE AND R.S.V.P. ANYWAY.
C			OR THE QUEUE MAY BE SHUTTING DOWN.
C
			CALL BUILD_RESP(SYS_STATUS,
	1	    	     %VAL(BUF.BUF$L_NRB),
	1		     %REF(BUF.BUF$S_MSG),	! PASS AS STRUCTURE
	1		     %VAL(RESP_ADDR))		! BUILD THE RESPONSE

	   endif		
	ENDIF

C
C ISSUE A NEW NETWORK READ 
C
1000	CONTINUE
	CALL NET_READ(%VAL(BUF.BUF$L_NRB)) ! ISSUE A NEW QIO FOR NEXT READ
	RETURN
	END
