	PROGRAM ROUTE_TESTER
	IMPLICIT NONE
C
C This program is a simple program that reads SYS$INPUT for
c message id's that are to be read/written by this application.
c By reading sys$input, this process can be ran multiple times
c via a batch command file.  The batch command file might look
c like the following (data records are defined below):
c
c $SET PROCESS/NAME="TEST1"
C $RUN MA_Q$PROD:ROUTE_TESTER
C W 0100 MESSAGEID_1
C R 0100 MESSAGEID_2
C W 0200 MESSAGEID_3
c $EOD
C
C The maximum number or message id's is 10 and the largest message
c size is 2048 bytes (excluding the header).
c
c The data records are defined as:
c
c  STRUCTURE/DATAREC/
C	CHARACTER*1 R_W	! READ/WRITE CODE 
C	CHARACTER*4 SIZE ! SIZE OF MESSAGE
C	CHARACTER*16 MID ! MESSAGE ID
C  END STRUCTURE
C

	INTEGER I			! Loop variable
	INTEGER MAXIDS		! MAXIMUM NUMBER OF MESSAGE IDS TO HANDLE
	PARAMETER (MAXIDS=10)
	INTEGER*4 SIZE(MAXIDS)	! SIZE OF EACH MESSAGE ID
	INTEGER*4 MAXMSGSIZE
	PARAMETER (MAXMSGSIZE=2048)	! Largest message to/from queue
	CHARACTER*1 R_W(MAXIDS)	! Direction (read/write)
	CHARACTER*16 MID(MAXIDS)	! Message ids
	INTEGER*4 IDCOUNT	! Actual number of message ids
	INTEGER*4 SYS_STATUS	! System service status messages
	INTEGER*4 INDEX(MAXIDS) ! Returned indexes from the connects
	INTEGER*4 RNA(2)	! Read not acknowleged pointer
C
c
	INCLUDE 'MA_Q$DEF:QUEUE.TLB(QEXDEF)/LIST'
	INCLUDE 'MA_Q$DEF:QUEUE.TLB(HDRDEF)/LIST'
	INCLUDE 'MA_Q$DEF:QUEUE.TLB(QUECONST)/LIST'
	INCLUDE 'MA_Q$DEF:QUEUE.TLB(QUEMSG)/LIST'
c
	RECORD /QEXDEF/Q_BLOCK	! EXIT BLOCK AND P0 EXPANDED REGION
C
	STRUCTURE/WRITEREC/
		CHARACTER*(MAXMSGSIZE) MSG ! largest record
	END STRUCTURE
	RECORD/WRITEREC/WRITEREC
C
	STRUCTURE/READREC/
		RECORD/HDRDEF/HDR
		CHARACTER*(MAXMSGSIZE) MSG ! Largest record
	END STRUCTURE
	RECORD/READREC/READREC
C
	CHARACTER*23 TIME_DELAY
	PARAMETER (TIME_DELAY= '0 00:01:00')	! ONE MINUTE
	INTEGER*4 RETRY_TIMER(2)	! QUAD BINARY TIME
	INTEGER*4 SYS$BINTIM
	INTEGER*4 SYS$SETIMR
	INTEGER*4 SYS$WAITFR

	DO I=1,MAXMSGSIZE		! load write buffer
		WRITEREC.MSG(I:I)='-'
	ENDDO
C
C ATTACH TO THE QUEUE
C
	SYS_STATUS=ATTACH_QE(Q_BLOCK)  ! Attach to the default queue
	IF(.NOT. SYS_STATUS)THEN
		TYPE *,' Unable to attach to the queue'
		CALL SYS$EXIT(%VAL(SYS_STATUS))
	ENDIF
C
C READ THE MESSAGE ID'S, ETC.
C
	IDCOUNT=1
50	READ(*,100,END=400)R_W(IDCOUNT),SIZE(IDCOUNT),MID(IDCOUNT)
100	FORMAT(A1,1X,I4,1X,A16)
C
	IF(R_W(IDCOUNT) .EQ. 'R' )THEN
	  SYS_STATUS=CONNECT_READ(Q_BLOCK,MID(IDCOUNT),1,INDEX(IDCOUNT))
	  IF(.NOT. SYS_STATUS)THEN
		TYPE *,' Unable to connect to queue for id: ',
	1		MID(IDCOUNT)
		CALL SYS$EXIT(%VAL(SYS_STATUS))
	  ELSE
		TYPE *,' MID: ',MID(IDCOUNT),' connected, index:',INDEX(IDCOUNT)
	  ENDIF
	ELSE
	  SYS_STATUS=GET_MID_INDEX(Q_BLOCK,MID(IDCOUNT),INDEX(IDCOUNT))
	  IF(.NOT. SYS_STATUS)THEN
		TYPE *,' Unable to get message id index for id: ',
	1		MID(IDCOUNT)
		CALL SYS$EXIT(%VAL(SYS_STATUS))
	  ELSE
		TYPE *,' MID: ',MID(IDCOUNT),' index:',INDEX(IDCOUNT)
	  ENDIF
	ENDIF
	IF(SIZE(IDCOUNT) .GT. MAXMSGSIZE)SIZE(IDCOUNT)=MAXMSGSIZE
	IDCOUNT=IDCOUNT+1	! Incr. number of ids we are working with
	IF(IDCOUNT .GT. MAXIDS)GO TO 400	! Done
	GO TO 50
C
C ALL MESSAGES SET UP
C
400	IDCOUNT=IDCOUNT-1
	IF(IDCOUNT .EQ. 0)THEN
		TYPE *,' NO MESSAGE IDS SPECIFIED, EXITING'
		CALL SYS$EXIT
	ENDIF

C
C WE MUST NOW WRITE MESSAGES TO THE QUEUE AND THEN READ
C ANY THAT WERE SPECIFIED.
C
500	DO 800 I=1,IDCOUNT
	  IF(R_W(I) .EQ. 'R')THEN	! READ?
		SYS_STATUS=READ_QREC(Q_BLOCK,
	1		MID(I),
	1		READREC,
	1		MAXMSGSIZE+HDR$K_SIZ,
	1		RNA,INDEX(I))
		IF(.NOT. SYS_STATUS)THEN	! READ OK?
		  IF(SYS_STATUS .EQ. %LOC(QUE_NOMESS))GO TO 800  ! Empty queue?
		  TYPE *,' Unable to read: ',MID(I)
		  TYPE *,' Taking out of scan list'
		  TYPE *,' STATUS: ',SYS_STATUS
		  R_W(I) = 'X'
		ELSE
		  TYPE *,' MID: ',MID(I),' read, ',
	1		' Message size: ',READREC.HDR.HDR$W_LEN
		  SYS_STATUS=ACK_READ(Q_BLOCK,
	1			MID(I),
	1			RNA,INDEX(I))
		  IF(.NOT. SYS_STATUS)THEN
			TYPE *,' Unable to acknowlege: ',MID(I)
			CALL SYS$EXIT(%VAL(SYS_STATUS))
		  ENDIF
		ENDIF
	ELSEIF(R_W(I) .EQ. 'W')THEN	! WRITE?
		SYS_STATUS=WRITE_QREC(Q_BLOCK,
	1			MID(I),
	1			WRITEREC,
	1			SIZE(I),
	1			0,INDEX(I))
		IF(.NOT. SYS_STATUS)THEN
		   TYPE *,' Bad write status on ID: ',MID(I)
		   TYPE *,' Status= ',SYS_STATUS
		   R_W(I)='X'  ! Remove from loop
		ENDIF
	ENDIF
800	CONTINUE

C
C loop complete,  wait a while and read/write again.
c 
	SYS_STATUS = SYS$BINTIM(TIME_DELAY,RETRY_TIMER)
	IF(.NOT. SYS_STATUS)THEN
		  CALL SYS$EXIT(%VAL(SYS_STATUS))
	ENDIF
c
	SYS_STATUS = SYS$SETIMR(%VAL(1),	! EVENT FLAG
	1		%REF(RETRY_TIMER),	! TIMER VALUE
	1		, 		! AST ADDRESS
	1		)		 ! REQUEST ID 
	IF(.NOT. SYS_STATUS)THEN
		  CALL SYS$EXIT(%VAL(SYS_STATUS))  ! FATAL BUG ERROR
	ENDIF
C
	SYS_STATUS=SYS$WAITFR(%VAL(1))
	IF(.NOT. SYS_STATUS)THEN
		CALL SYS$EXIT(%VAL(SYS_STATUS))
	ELSE
		GO TO 500
	ENDIF
	END

