	INTEGER*4 FUNCTION ACQUIRE(CAP)
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&ACQUIRE\&
C .nf
C .x ACQUIRE>Defined
C Source:ACQUIRE_RELEASE.FOR
C Designer           :DAVE MAX
C Author             :dave max, E. lakia, Sue Gelatka
C Date of last update:02-DEC-85
C Revision level     :1.2
C  
C .C ;Formal Parameter List
C Receives:
C  
C    CAP                        INTEGER*4
C                               EVENT FLAG ASSOCIATED WITH RESOURCE TO BE
C                               CAPTURED
C
C Returns:
C  
C    ACQUIRE                    INTEGER*4                                       
C                               RETURN STATUS - SS$_NORMAL INDICATES 
C                               SUCCESSFUL CAPTURE
C  
C Accesses common(s):
C  
C Accesses file(s):
C  
C Other modules referenced:
C  
C .X OPR_LOG_MSG>Referenced
C    OPR_LOG_MSG                LOGS ERROR TO OPERATOR CONSOLE    
C                               (found in library[2BOF.PROD.LIBRARY])
C
C .X APP_LOG_MSG_INT>Referenced
C    APP_LOG_MSG_INT            LOGS ERROR WITH EVENT FLAG NUMBER
C                               TO OPERATOR CONSOLE
C
C .SK
C .fill
C .SK
C Description:
C .sk
C	This subroutine works in conjunction with release subroutine
c to capture and release a controlled resource. This is done via a global 
c event flag that the caller has associated to a global event flag cluster.
C
C 	CALLING SEQUENCE:
C	    INTEGER*4 	    ACQUIRE
C 	    INTEGER*4	    SYS_STATUS
C 	    INTEGER*4	    CAP
C 	SYS_STATUS = ASSOCIATE GLOBAL EVENT FLAG CLUSTER
C 	SYS_STATUS = ACQUIRE (CAP)
C 	IF (.NOT. SYS_STATUS) THEN
C
C 		ERROR
C
C       ERROR CODES:
C	SS$_DISCONNECT -
C	    USER HAS NOT CALLED EXIT HANDLER,
C	    ACQUIRE REQUEST IS ABORTED
C	SS$_ILLEFC -
C	    EVENT FLAG NOT WITHIN RANGE
C end.doc ****************************** end.doc  
 
	IMPLICIT NONE
	INCLUDE '($SSDEF)'
	! define input parameter
	INTEGER*4	CAP                ! THE VALUE OF THE EVENT FLAG

	! declare functions
	INTEGER*4	SYS_STATUS	   ! STATUS FROM SYSTEM SERVICES
	INTEGER*4	SYS$CLREF	   ! clear event flag
	INTEGER*4	SYS$WAITFR	   ! wait for event flag

	INTEGER*4	DESBLK(4)	   ! EXIT CONTROL BLOCK
	INTEGER*4	CAP_EFN		   ! LONGWORD TO STORE SET EVENT FLAGS

	COMMON /EFN_EXIT_COMMON/ DESBLK,CAP_EFN


	IF(CAP .GE. 64 .AND. CAP .LE. 95) THEN  ! IS GLOBAL EVENT FLAG
						! WITHIN RANGE
	  GOTO 5
	ELSE
	  CALL APP_LOG_MSG_INT(SS$_ILLEFC,
     1         ' ERROR EFN IS NOT BETWEEN 64 AND 95, EFN: ',
     1         40,CAP)
	  ACQUIRE=SS$_ILLEFC	! RETURN ERROR CODE
	  RETURN
	ENDIF
	
    5   IF(DESBLK(2) .EQ. 0) THEN         ! HAS CALLER SET UP EXIT HANDLER
	  ACQUIRE = SS$_DISCONNECT
	  CALL OPR_LOG_MSG(SS$_DISCONNECT,
     1      'MUST SET UP EXIT HANDLER: SYS_STATUS = EFN_EXIT(CAP_EFN)',
     1      56)
	  RETURN
	ENDIF

10	ACQUIRE = SYS$CLREF(%VAL(CAP))          ! try to capture someone

	IF (ACQUIRE .EQ. SS$_WASSET) THEN	! we are captured
D	   CALL APP_LOG_MSG_INT(SS$_MSGNOTFND,
D	1	' CAPTURED RESOURCE CONTROLLED BY EVENT FLAG: ',46,
D	1	CAP)
           ACQUIRE = SS$_NORMAL		! okay to begin processing
C
C       SET GLOBAL EVENT FLAG CAP IN THE LONGWORD CAP_EFN
C
	   CAP_EFN = IBSET(CAP_EFN,CAP - 64)    
D	   CALL APP_LOG_MSG_INT(1,
D     1          'IN CAP_EFN SET EVENT FLAG NO: ',
D     1          30,CAP)
	ELSE IF (ACQUIRE .EQ. SS$_WASCLR) THEN  ! already captured
	   CALL APP_LOG_MSG_INT(ACQUIRE,
	1	' WAITING FOR RESOURCE, EVENT FLAG: ',34,
	1	CAP)
	   ACQUIRE = SYS$WAITFR(%VAL(CAP)) 	! wait for owner to release
	   GO TO 10		                ! it and wake us up
        ENDIF

	RETURN
	END
	INTEGER*4 FUNCTION RELEASE(CAP)
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&RELEASE\&
C .nf
C .x RELEASE>Defined
C Source:ACQUIRE_RELEASE.FOR
C Designer           :DAVE MAX
C Author             :Dave Max,E. lakia, Sue Gelatka
C Date of last update: 02-DEC-85
C Revision level     :1.2
C  
C .C ;Formal Parameter List
C Receives:
C
C    CAP                        INTEGER*4
C                               EVENT FLAG ASSOCIATED WITH RESOURCE TO BE
C                               RELEASED
C   
C Returns:
C  
C    RELEASE                    INTEGER*4                                       
C                               RETURN STATUS - SS$_NORMAL INDICATES
C                               SUCCESSFUL RELEASE
C  
C Accesses common(s):
C  
C Accesses file(s):
C  
C Other modules referenced:
C  
C .X OPR_LOG_MSG>Referenced
C    OPR_LOG_MSG                LOGS ERROR TO OPERATOR CONSOLE    
C                                (found in library:[2BOF.PROD.LIBRARIES])
C
C .X APP_LOG_MSG_INT>Referenced
C    APP_LOG_MSG_INT            LOGS ERROR WITH EVENT FLAG NUMBER
C                               TO OPERATOR CONSOLE
C
C .SK
C .fill
C .SK
C Description:
C .sk
C 	This subroutine works in conjunction with acquire subroutine to
c capture and release a controlled resource. this is done via a global
c event flag that the caller has associated to a global event flag cluster 
c with.
C 
C 	CALLING SEQUENCE:
C	   INTEGER*4	   RELEASE
C 	   INTEGER*4	   SYS_STATUS
C 	   INTEGER*4       CAP
C 	SYS_STATUS = RELEASE (CAP)
C 	IF (.NOT. RELEASE) THEN
C
C 		ERROR
C
C	ERROR CODES:     
C	SS$_ILLEFC -
C	    EVENT FLAG NOT WITHIN RANGE
C end.doc ****************************** end.doc  

	IMPLICIT NONE
	INCLUDE '($SSDEF)'                      ! CONDITION RETURNS FROM SYSTEM
	INTEGER*4	CAP	                ! THE VALUE OF THE EVENT FLAG

	! declare functions
	INTEGER*4	SYS$SETEF		! set event flag

	INTEGER*4	DESBLK(4)		! EXIT CONTROL BLOCK
	INTEGER*4	CAP_EFN			! LONGWORD TO STORE SET
						! EVENT FLAGS

	COMMON /EFN_EXIT_COMMON/ DESBLK,CAP_EFN

	IF(CAP .GE. 64 .AND. CAP .LE. 95) THEN  ! IS GLOBAL EVENT FLAG
						! WITHIN RANGE
	  GOTO 5
	ELSE
	  CALL APP_LOG_MSG_INT(SS$_ILLEFC,
     1         ' ERROR EFN IS NOT BETWEEN 64 AND 95, EFN: ',
     1         40,CAP)
	  RELEASE=SS$_ILLEFC	! RETURN ERROR STATUS
	  RETURN
	ENDIF
    5   CONTINUE
	RELEASE = SYS$SETEF(%VAL(CAP)) ! release resource
	IF (.NOT. RELEASE) THEN        ! able to 
	    CALL APP_LOG_MSG_INT(RELEASE,
	1	'Unable to release resource controlled by EFN: ',
	1	47,CAP)
	    RETURN
	ENDIF
D	CALL APP_LOG_MSG_INT(RELEASE,
D	1	' RELEASED RESOURCE CONTROLLED BY EVENT FLAG: ',
D	1	41,CAP)
C
C	CLEAR GLOBAL EVENT FLAG CAP IN THE LONGWORD CAP_EFN
C
	CAP_EFN = IBCLR(CAP_EFN,CAP - 64)
D	CALL APP_LOG_MSG_INT(1,
D     1       'IN CAP_EFN CLEARED EVENT FLAG NO: ',
D     1       34,CAP)
	RETURN	
	END
	INTEGER*4 FUNCTION EFN_EXIT(CAP)
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&EFN_EXIT\&
C .nf
C .x EFN_EXIT>Defined
C Source:ACQUIRE_RELEASE_EXIT.FOR
C Designer           :SUZANNE GELATKA
C Author             :SUZANNE GELATKA
C Date of last update: 02-DEC-85
C Revision level     :1.2
C  
C .C ;Formal Parameter List
C Receives:
C
C    CAP			INTEGER*4
C				DUMMY ARGUMENT WHICH HAS NO VALUE WHEN
C				PASSED.
C Returns:
C
C    EFN_EXIT                   INTEGER*4 FUNCTION
C                               RETURN SYSTEM SERVICE STATUS FROM THE
C                               DECLARE EXIT HANDLER SYSTEM SERVICE CALL
C 	
C Accesses common(s):
C  
C Accesses file(s):
C  
C Other modules referenced:
C
C .SK
C .fill
C .SK
C Description:
C .sk
C .FILL
C THIS SUBROUTINE SETS UP AN EXIT HANDLER "EFN_EXIT_HAN" TO BE CALLED
C WHEN THIS PROCESS EXITS.
C
C .NOFILL
C CALLING SEQUENCE:
C     INTEGER*4 EFN_EXIT
C     INTEGER*4 SYS_STATUS
C     INTEGER*4 CAP_EFN
C     SYS_STATUS = EFN_EXIT(CAP)
C .fill
c 	The status returned is from the system service "DCLEXH"
c system call.  It should always be normal.&.but.
c 
C end.doc ***************************** end.doc  
	IMPLICIT NONE
	INCLUDE'($SSDEF)'
	INTEGER*4 SYS_STATUS	      ! STATUS
	INTEGER*4 SYS$DCLEXH          ! DECLARE SYSTEM EXIT HANDLER 
				      ! SYSTEM SERVICE
	INTEGER*4 EXIT_STATUS        ! EXIT STATUS
	INTEGER*4 DESBLK(4)           ! EXIT CONTROL BLOCK
	INTEGER*4 CAP
	INTEGER*4 CAP_EFN             ! LONG WORD TO STORE SET EVENT FLAGS
	EXTERNAL EFN_EXIT_HAN

	COMMON /EFN_EXIT_COMMON/ DESBLK,CAP_EFN
C
C	CALL SYSTEM SERVICE TO LINK IN THE EXIT HANDLER IN
C	USER STATE
C
	DESBLK(2) = %LOC(EFN_EXIT_HAN) ! ADDRESS OF EXIT HANDLER
	DESBLK(3) = 1                  ! NUMBER OF ARGUMENTS
	DESBLK(4) = %LOC(EXIT_STATUS)
	EFN_EXIT = SYS$DCLEXH(DESBLK)  ! DECLARE EXIT HANDLER
	RETURN
	END

	SUBROUTINE EFN_EXIT_HAN(EXIT_STATUS)
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&EFN_EXIT_HAN\&
C .nf
C .x EFN_EXIT_HAN>Defined
C Source:ACQUIRE_RELEASE_EXIT.FOR
C Designer           :SUZANNE GELATKA
C Author             :SUZANNE GELATKA
C Date of last update: 4/23/86
C Revision level     :1.2
C  
C .C ;Formal Parameter List
C Receives:
C
C    EXIT_STATUS                INTEGER*4
C                               SYSTEM EXIT STATUS
C
C Returns:
C	
C Accesses common(s):
C  
C Accesses file(s):
C  
C Other modules referenced:
C
C .SK
C .fill
C .SK
C Description:
C .sk
C 	THIS SUBROUTINE IS CALLED BY THE SYSTEM WHEN THE PROCESS EXITS.
C THIS SUBROUTINE IS A USER EXIT HANDLER.  IT WAS SET UP TO BE
C CALLED BY THE FUNCTION EFN_EXIT WHICH PLACED THE ADDRESS OF THIS ROUTINE
C INTO THE EXIT HANDLER DESCRIPTOR BLOCK. THIS SUBROUTINE
C CLEARS THE EVENT FLAGS THAT WERE SET BY THE SUBROUTINE ACQUIRE
C AND WERE NOT RELEASED BY THE RELEASE SUBROUTINE.  THE FLAGS THAT
C NEED TO BE RELEASED ARE SET IN A BIT MASK 
C IN THE LONGWORD "CAP_EFN".
c 	The basic purpose of this routine is to guarantee that no
c resource that is controlled by the global event flags is left
c captured by any process in the event of abnormal exit or
c programming bug.
c 
C end.doc ****************************** end.doc  
	IMPLICIT NONE
	INCLUDE'($SSDEF)'
	INTEGER*4 EXIT_STATUS         ! SYSTEM EXIT REASON
	INTEGER*4 DESBLK(4)	      ! EXIT CONTROL BLOCK
	INTEGER*4 CAP_EFN	      ! LONGWORD TO STORE GLOBAL
				      ! EVENT FLAGS
	INTEGER   I
	INTEGER*4 EF
	INTEGER*4 STATUS
	INTEGER*4 RELEASE
	COMMON /EFN_EXIT_COMMON/ DESBLK,CAP_EFN
C
C	CLEAR ALL THE EVENT FLAGS THAT ARE SET
C
	IF(EXIT_STATUS .NE. SS$_NORMAL)
	1 CALL OPR_LOG_MSG(EXIT_STATUS,' EVENT FLAG EXIT HANDLER CALLED',31)
	DO 10 I = 0,31
	  IF(BTEST(CAP_EFN,I)) THEN
	    EF = I + 64
	    STATUS = RELEASE(EF)
             CALL APP_LOG_MSG_INT(1,
     1       'EXIT HANDLER FROM ABNORMAL EXIT CLEARED EVENT FLAG NO: ',
     1       55,EF)
	  ENDIF
   10   CONTINUE
	IF(EXIT_STATUS .NE. SS$_NORMAL)
	1	CALL LIB$SIGNAL(%VAL(ss$_normal))  ! SIGNAL FOR TRACEBACK
	RETURN
	END
