	OPTIONS/EXTEND_SOURCE
	INTEGER*4 FUNCTION SUB_TIMES_VMS ( ABS_TIME, DELTA_TIME, TIME_DIFF )
	IMPLICIT NONE
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&SUB_TIMES_VMS\&
C .nf
C .x SUB_TIMES_VMS>Defined
C Source:SUB_TIMES_VMS.FOR
C TO BECOME PART OF LIBRARY:[2BOF.PROD.LIBRARIES]USER
C Designer           :ROBERT F. READ
C Author             :R. F. READ
C Date of last update:02-DEC-1985
C Revision level     :1.0
C  
C .C ;Formal Parameter List
C Receives:
C  
C    ABS_TIME                   CHARACTER*23                            
C                               ABSOLUTE TIME - FROM WHICH DELTA TIME IS TO BE 
C                               SUBTRACTED; TIME IS IN STANDARD VAX/VMS TIME 
C                               FORMAT - 'dd-mmm-yyyy hh:mm:ss.cc'
C  
C    DELTA_TIME                 CHARACTER*16                                  
C                               DELTA TIME - TO BE SUBTRACTED FROM ABSOLUTE TIME
C                               TIME IS IN STANDARD VAX/VMS FORMAT FOR DELTA TIME
C                               'dddd hh:mm:ss.cc'
C  
C Returns:
C  
C    TIME_DIFF                  CHARACTER*23                                      
C                               DIFFERENCE OF ABS_TIME AND DELTA_TIME                    
C                               - IN STANDARD VAX/VMS FORMAT                      
C 
C    STATUS                     INTEGER*4 
C                               SS$_NORMAL INDICATES SUCCESSFUL COMPLETION;
C                               ANY STATUS OTHER THAN SS$_NORMAL SHOULD BE
C                               REGARDED AS AN ERROR CONDITION
C  
C Accesses common(s):
C  
C Accesses file(s):
C  
C Other modules referenced:
C  
C .X SYS$BINTIM>Referenced
C    SYS$BINTIM                 CONVERTS TIME STRING TO INTERNAL BINARY FORMAT    
C                                (found in library:SYSTEM SERVICES)
C  
C .X SYS$ASCTIM>Referenced
C    SYS$ASCTIM                 CONVERT INTERNAL FORMAT TIME TO STANDARD VAX/VMS  
C                               (ASCII) FORMAT                                    
C                                (found in library:SYSTEM SERVICES)
C  
C .X LIB$ADDX>Referenced
C    LIB$ADDX                   SUBTRACTS TWO QUADWORD INTEGER VALUES                  
C                                (found in library:RUN-TIME)
C 
C .X OPR_LOG_MSG>Referenced
C    OPR_LOG_MSG                LOGS APPROPRIATE ERROR MESSAGE TO OPERATOR CONSOLE
C                                (found in library:USER)
C .SK
C .fill
C .SK
C Description:
C .sk    
C 
CUSAGE:  STATUS = SUB_TIMES_VMS ( ABS_TIME, DELTA_TIME, TIME_DIFF )
C 
C 	This function takes the difference of two VAX/VMS time strings - 
C one of standard 23-character format and one of standard 16-character
C delta time format.  In order to perform the subtraction the time
C strings must first be converted to internal binary format, then subtracted
C as quadword values, and finally the result is converted back to
C standard VAX/VMS format.  If an error condition is encountered during
C any of the conversions or during the subtraction, an error message is 
C logged to the operators console and an error status is returned to
C the calling program.  When an error condition is encountered process
C control is returned to the calling program immediately. 
C end.doc ****************************** end.doc  

	! declare input variables
	   CHARACTER*23	ABS_TIME	! INPUT TIME IN VAX/VMS FORMAT
	   CHARACTER*16	DELTA_TIME	! INPUT TIME IN VAX/VMS DELTA FORMAT
	   CHARACTER*23	TIME_DIFF	! RETURNED TIME IN VAX/VMS FORMAT

	! Local variables
	INTEGER*4 	ABS_TIME_B(2)	! Time 1 - internal format
	INTEGER*4 	DELTA_TIME_B(2)	! Time 2 - internal format
	INTEGER*4 	TIME_DIFF_B(2)	! Time total - internal format

	! Integer functions
	INTEGER*4 	SYS$BINTIM	! System Service to convert time to
					!    internal format
	INTEGER*4 	SYS$ASCTIM	! System Service to convert time to
					!    VAX/VMS (ASCII) format
	INTEGER*4 	LIB$ADDX	! Run-time function to subtract two 
					!    quadwords


C
C	Convert time strings to internal format
C
	SUB_TIMES_VMS = SYS$BINTIM ( ABS_TIME, ABS_TIME_B )
	IF ( .NOT. SUB_TIMES_VMS ) THEN
	   CALL APP_LOG_MSG_CHAR ( SUB_TIMES_VMS, '%SUB_TIMES_VMS:  unable '//
	1  'to convert absolute time value = ', 57, ABS_TIME, LEN(ABS_TIME) )
	   RETURN
	END IF

	SUB_TIMES_VMS = SYS$BINTIM ( DELTA_TIME, DELTA_TIME_B )
	IF ( .NOT. SUB_TIMES_VMS ) THEN
	   CALL APP_LOG_MSG_CHAR ( SUB_TIMES_VMS, '%SUB_TIMES_VMS:  unable '//
	1  'to convert delta time value = ', 54, DELTA_TIME, LEN(DELTA_TIME) )
	   RETURN
	END IF

C
C	Add times - because delta time is stored as a negative value
C	   the difference is determined by adding the negative
C	   delta time to the absolute time
C
	SUB_TIMES_VMS = LIB$ADDX ( ABS_TIME_B, DELTA_TIME_B, TIME_DIFF_B )
	IF ( .NOT. SUB_TIMES_VMS ) THEN
	   CALL OPR_LOG_MSG ( SUB_TIMES_VMS, '%SUB_TIMES_VMS:  unable to add '//
	1  'delta time to absolute time', 58 )
	   RETURN
	END IF

C
C	Convert time difference to standard format
C
	SUB_TIMES_VMS = SYS$ASCTIM ( , TIME_DIFF, TIME_DIFF_B, )
	IF ( .NOT. SUB_TIMES_VMS ) THEN
	   CALL OPR_LOG_MSG ( SUB_TIMES_VMS, '%SUB_TIMES_VMS:  unable to '//
	1  'convert internal time diff. to standard VMS format', 77 )
	   RETURN
	END IF


	RETURN
	END
