	INTEGER*4 FUNCTION SUBMIT_JOB(JOB,QUEUE,LOG,FILE)
	implicit none
C.begin.doc ************************** begin.doc
C  
C .c ;MODULE
C .c ;^&SUBMIT_JOB\&
C .nf
C .x SUBMIT_JOB>Defined
C Source:SUBMIT_JOB.FOR
C TO BECOME PART OF LIBRARY:INLAND
C Designer           :EARL LAKIA
C Author             :EARL LAKIA
C             Inland Steel
C             Process Automation Department
C             3210 Watling St.  MS 2-465
C             East Chicago, IN  46312
C  
C Date of last update:12-NOV-1986 08:51:08.52
C Revision level     :1.0
C  
C .C ;Formal Parameter List
C Receives:
C  
C    JOB                        CHARACTER*(1 TO 39)                               
C                               JOB NAME TO BE GIVEN TO JOB BEING SUBMITTED, THIS 
C                               MAY BE IN THE RANGE OF 1 TO 39 CHARACTERS.        
C  
C    QUEUE                      CHARACTER (1 TO 31)                               
C                               NAME OF QUEUE THAT THE JOB IS TO BE SUBMITTED     
C                               TO. SUCH AS: SYS$BATCH.                           
C  
C    LOG                        CHARACTER*(*)                                     
C                               FILE NAME OF THE LOG FILE FOR THE BATCH JOB. IF   
C                               NO LOG FILE IS DESIREC, THEN A CHARACTER          
C                               DESCRIPTOR OF ZERO LENGTH MAY BE USED             
C                               TO ELIMINATE THE LOG FILE.                        
C  
C    FILE                       CHARACTER*(*)                                     
C                               COMMAND FILE THAT CONTAINS THE ACTUAL JOB TO      
C                               BE SUBMITTED.                                     
C  
C Returns:
C  
C    SUBMIT_JOB                 INTEGER*4                                         
C                               RETURN STATUS FROM EITHER THE SYSTEM SERVICE      
C                               OR THE RETURN VALUE FROM THE REQUEST.             
C  
C Accesses common(s):
C  
C Accesses file(s):
C  
C Other modules referenced:
C .SK
C .fill
C .SK
C Description:
C .sk    
C 	This subroutine allows the caller to submit a job to a desired
C queue.  This function does the same function as the following
C DCL commands:
C .sk
C SUBMIT/QUEUE=SYS$BATCH/LOG=logfile/JOB_NAME=jobname job file
C .sk
C 	The user has the option of not having a log file by
C building a null length descriptor.  The calling sequence is:
C .sk 2
C .nofill
C 	INTEGER*4 SUMBIT_JOB
C 	INTEGER*4 STATUS
C 	CHARACTER*9 QUEUE
C 	CHARACTER*4 JOB
C 	CHARACTER*8 LOG
C 	CHARACTER*8 FILE
C 	INTEGER*4 NULLCHAR(2)
C C
C 	QUEUE='SYS$BATCH'
C 	JOB='EARL'
C 	LOG='EARL.LOG'
C 	FILE='EARL.COM'
C C
C 	STATUS=SUBMIT_JOB(JOB,QUEUE,LOG,FILE)
C C
C C SUBMIT WITHOUT LOG FILE
C C
C 	NULLCHAR(1)=0	! NULL LENGTH CHAR. DESCRIPTOR
C C
C 	STATUS=SUBMIT_JOB(JOB,QUEUE,NULLCHAR,FILE)
C .FILL
C 	
C 	
C end.doc ****************************** end.doc  

	INCLUDE '($SSDEF)'
	INCLUDE '($SJCDEF)'
	CHARACTER*(*) JOB	! USER JOB NAME
	CHARACTER*(*) QUEUE	! QUEUE NAME
	CHARACTER*(*) LOG	! LOG FILE NAME
	CHARACTER*(*) FILE	! FILE TO BE SUBMITTED
C
	STRUCTURE/JBCITM/	! JBC ITEM LIST
		INTEGER*2 BUFLEN	! BUFFER LENGTH
		INTEGER*2 ITEM		! ITEM CODE
		INTEGER*4 BUFADD	! BUFFER ADDRESS
		INTEGER*4 RETLTH	! RETURN LENGTH
	END STRUCTURE
C
	INTEGER*4 SYS_STATUS		! SYSTEM SERVICE STATUS
	INTEGER*4 SYS$SYNCH		! I/O SYNC
	INTEGER*4 SYS$SNDJBCW		! SEND TO JOB CONTROLLER
C
	STRUCTURE/STATUS/
		INTEGER*4 COND		! STATUS
		INTEGER*4 UNUSED	! NOT USED
	END STRUCTURE
C
	RECORD/STATUS/STATUS
C
	RECORD/JBCITM/CREATE(5)	! CREATE JOB ITEM LIST
	RECORD/JBCITM/CLOSE(2)	! CLOSE JOB REQUEST
	INTEGER I

C
C COPY USER PARAMETERS
C
	CREATE(1).ITEM=SJC$_QUEUE	! QUEUE NAME
	IF(LEN(QUEUE) .LE. 31)THEN	! QUEUE NAME TOO LONG
	 	CREATE(1).BUFLEN=LEN(QUEUE)	! LENGTH
	ELSE
		CREATE(1).BUFLEN=31
	ENDIF
	CREATE(1).BUFADD=%LOC(QUEUE)	! ADDRESS
	CREATE(1).RETLTH=0
C
	IF(LEN(LOG) .EQ. 0)THEN
		CREATE(2).ITEM=SJC$_NO_LOG_SPECIFICATION  ! NO LOG FILE
		CREATE(2).BUFLEN=0
		CREATE(2).BUFADD=0
		CREATE(2).RETLTH=0
	ELSE
		CREATE(2).ITEM=SJC$_LOG_SPECIFICATION ! LOG FILE SPECIFICATION
	 	CREATE(2).BUFLEN=LEN(LOG)
		CREATE(2).BUFADD=%LOC(LOG)
		CREATE(2).RETLTH=0
	ENDIF
C
	CREATE(3).ITEM=SJC$_FILE_SPECIFICATION ! FILE TO BE SUBMITTED
 	CREATE(3).BUFLEN=LEN(FILE)
	CREATE(3).BUFADD=%LOC(FILE)
	CREATE(3).RETLTH=0
C
	CREATE(4).ITEM=SJC$_JOB_NAME	! JOB NAME
	IF(LEN(JOB) .GT. 39)THEN		! JOB NAME TOO LONG?
	 	CREATE(4).BUFLEN=39
	ELSE
		CREATE(4).BUFLEN=LEN(JOB)
	ENDIF
	CREATE(4).BUFADD=%LOC(JOB)
	CREATE(4).RETLTH=0

	CREATE(5).ITEM=0		! TERMINATOR
 	CREATE(5).BUFLEN=0
	CREATE(5).BUFADD=0
	CREATE(5).RETLTH=0
C
C
D	DO 100 I=1,5
D		WRITE(*,10)CREATE(I).ITEM,
D	1		CREATE(I).BUFLEN,
D	1		CREATE(I).BUFADD,
D	1		CREATE(I).RETLTH
DC
D10		FORMAT(' ITEM:       ',Z4.4,/,
D	1	       ' BUFLEN:     ',Z8.8,/,
D	1	       ' BUFADD:     ',Z8.8,/,
D	1	       ' RETLTH:     ',Z8.8)
D100	CONTINUE
C
	SUBMIT_JOB=SYS$SNDJBCW(,	! EFN (DEFAULT TO ZERO)
	1		%VAL(SJC$_ENTER_FILE), 	! FUNCTION CODE
	1		,			! NULL ARGUMENT PLACE HOLDER
	1		CREATE,			! ITEM LIST
	1		STATUS,			! RETURN STATUS
	1		,)			! AST, AST PARM. NOT USED
C
C
	IF(.NOT. SUBMIT_JOB)RETURN
C
	SYS_STATUS=SYS$SYNCH(,STATUS)	! WAIT UNTIL DONE
C
	SUBMIT_JOB=STATUS.COND  ! RETURN STATUS
	END
