#include descrip
#include ssdef
#include iodef
#include "nfbdef.h"

#define MAXITEMS 10
struct nfbdef {
    unsigned char nfb$b_fct;
    unsigned char nfb$b_flags;
    unsigned char nfb$b_database;
    unsigned char nfb$b_oper;
    long int nfb$l_srch_key;
    long int nfb$l_srch2_key;
    unsigned char nfb$b_oper2;
    unsigned char nfb$b_mbz1;
    short int nfb$w_cell_size;
    long int nfb$l_fldid[MAXITEMS];
};

struct nfbdef1 {
  union {
    long int nfb$l_param_id;
    struct {
      unsigned nfb$v_inx : 16;
      unsigned nfb$v_typ : 2;
      unsigned nfb$v_spare : 6;
      unsigned nfb$v_db : 8;
    } fields;
  } overlay;
};

struct nfbdef2 {
  union {
    long int nfb$l_lng_value;
    long int nfb$l_bit_value;
    struct {
      short int nfb$w_str_count;
      char nfb$b_str_text;
    } string;
  } overlay;
};

main()
{
  $DESCRIPTOR(Net,"Net:");
  struct nfbdef nfb = {	NFB$C_FC_SHOW, 0, NFB$C_DB_LLI, NFB$C_OP_EQL,
			NFB$C_WILDCARD, NFB$C_WILDCARD, 0, 0, 0,
			NFB$C_LLI_USR,
			NFB$C_LLI_PRC,
			NFB$C_LLI_PNN,
			NFB$C_LLI_RID,
			NFB$C_LLI_LLN,
			NFB$C_LLI_RLN,
			NFB$C_LLI_PNA,
			NFB$C_ENDOFLIST };
  struct dsc$descriptor nfbdesc = {sizeof(nfb), 0, 0, &nfb};

  struct {
    long int k1;
    long int k2;
    char ctx[NFB$C_CTX_SIZE];
  } key = {0,0};
  struct dsc$descriptor keydesc = {sizeof(key), 0, 0, &key};

  static unsigned char buf[128];
  struct dsc$descriptor bufdesc = {sizeof(buf), 0, 0, buf};

  int status, chan, iosb[2], retlen = 0, i;
  unsigned short int len;
  char *buff;
  char *errs[] = {	"unrecognized NFB$B_FCT value",
			"unrecognized NFB$B_DATABASE value",
			"The P1 buffer is invalid",
			"The P2 buffer is invalid",
			"The P3 buffer is invalid",
			"The P4 buffer is invalid",
			"The P5 buffer should not have been specified",
			"The P6 buffer should not have been specified",
			"unrecognized NFB$B_CELL value",
			"unrecognized NFB$B_OPER value",
			"unrecognized NFB$L_SRCH_KEY field i.d.",
			"unrecognized NFB$L_SRCH2_KEY field i.d.",
			"unrecognized NFB$B_OPER2 value",
			"undefined bits in NFB$B_FLAGS were not zero"
		};

  status = sys$assign (&Net, &chan, 0, 0);
  if (!( status & 1))
  {
    lib$signal(status);
    return(status);
  }

  while (status & 1)
  {
    status = sys$qiow (0,chan,IO$_ACPCONTROL,iosb,0,0,
			&nfbdesc,&keydesc,&retlen,&bufdesc,0,0);

    if (status & 1)
    {
      if (!(iosb[0] & 1))
      {
        status = iosb[0];
        switch (status)
        {
          case SS$_NOPRIV:
            /* bits are set in iosb[1] to indicate the missing privs */
            printf("No priv for attempted operation (%08X)\n",iosb[1]);
            break;
          case SS$_ILLCNTRFUNC:
            printf("Illegal control function (%08X)\n",iosb[1]);
            printf("%s\n",
		((iosb[1] < 1) || (iosb[1] > 14))?"?":errs[iosb[1] - 1]);
            break;
          case SS$_BADPARAM:
            printf("Field ID (%08X) not recognized\n",iosb[1]);
            break;
          case SS$_ENDOFFILE:
            printf("No entries found for search key (%08X)\n",iosb[1]);
            break;
        }
      }
    }
    if (!(status & 1))
    {
      return(status);
    }

    buff = buf;
    for (i = 0; nfb.nfb$l_fldid[i] != NFB$C_ENDOFLIST; ++i)
    {
      buff += print_param(nfb.nfb$l_fldid[i], buff);
    }
    printf("\n");
  }

  sys$dassgn(chan);
  if (status & 1) status = iosb[0];
  if (!( status & 1))
  {
    lib$signal(status);
    return(status);
  }

  return(1);
}

print_param(param, val)
  struct nfbdef1 param;
  struct nfbdef2 *val;
{
  switch (param.overlay.fields.nfb$v_typ)
  {
    case NFB$C_TYP_BIT:
      if (val->overlay.nfb$l_bit_value == -1)
        printf("unknown ");
      else
        printf("%d ",val->overlay.nfb$l_bit_value);
      return (4);
      break;
    case NFB$C_TYP_LNG:
      if (val->overlay.nfb$l_lng_value == -1)
        printf("unknown ");
      else
      {
        if (param.overlay.nfb$l_param_id == NFB$C_LLI_PNA)
          printf("%d.%d ",val->overlay.nfb$l_lng_value / 1024,
			  val->overlay.nfb$l_lng_value % 1024);
        else
          printf("%d ",val->overlay.nfb$l_lng_value);
      }
      return (4);
      break;
    case NFB$C_TYP_STR:
      printf("\"%.*s\" ",val->overlay.string.nfb$w_str_count,
			&(val->overlay.string.nfb$b_str_text));
      return(2 + val->overlay.string.nfb$w_str_count);
      break;
  }
}
