#define BIT unsigned
#define BYTE unsigned char
#define WORD short int
#define LONG long int
#define QUAD struct {unsigned:32; unsigned:32;}

/* *** MODULE $KFDDEF ***
 * 
 * KNOWN FILE DEVICE AND DIRECTORY BLOCK DEFINITIONS
 */
#define KFD$C_LENGTH 17	/* Length of fixed area of kfd entry */
#define KFD$S_KFDDEF 17
struct kfddef {
  LONG kfd$l_link;	/* Device, Directory, Extension (KFD) list link */
  LONG kfd$l_kfelist;	/* Ordered Known file entry list header */
  WORD kfd$w_size;	/* Size of block */
  BYTE kfd$b_type;	/* Structure type */
  BYTE kfd$b_spare;	/* spare */
  WORD kfd$w_refcnt;	/* Number of KFE's with same KFD */
  BYTE kfd$b_devlen;	/* Length of Device string */
  BYTE kfd$b_dirlen;	/* Length of Directory string */
  BYTE kfd$b_ddtstrlen;	/* Length of Device, Directory, Type (DDT) string */
  BYTE kfd$t_ddtstr;	/* Offset to DDT string */
};

 
/* *** MODULE $KFEDEF ***
 * 
 * KNOWN FILE ENTRY DEFINITIONS
 */
#define KFE$M_PROTECT 1
#define KFE$M_LIM 2
#define KFE$M_PROCPRIV 4
#define KFE$M_OPEN 8
#define KFE$M_HDRRES 16
#define KFE$M_SHARED 32
#define KFE$M_SHMIDENT 64
#define KFE$M_COMPATMOD 128
#define KFE$M_NOPURGE 256
#define KFE$M_ACCOUNT 512
#define KFE$M_WRITEABLE 1024
#define KFE$M_EXEONLY 2048
#define KFE$K_LENGTH 55	/* Length of fixed area of KFE entry */
#define KFE$C_LENGTH 55	/* Length of fixed area of KFE entry */
#define KFE$K_MAXLEN 94	/* Max KFE length (includes max filename) */
#define KFE$C_MAXLEN 94	/* Max KFE length (includes max filename) */
#define KFE$S_KFEDEF 55
#define KFE$S_PROCPRIV 8	/*   Process privilege mask */
struct kfedef {
  LONG kfe$l_hshlnk;	/*   Known file Hash table link */
  LONG kfe$l_kfelink;	/*   Ordered Known file entry list link */
  WORD kfe$w_size;	/*   Size of block */
  BYTE kfe$b_type;	/*   Structure type */
  BYTE kfe$b_hshidx;	/*   KFE hash table index (index into vector of HSHQ's) */
  LONG kfe$l_kfd;	/*   Device, Directory, Type block */
  union {
    WORD kfe$w_flags;	/*   Flags word */
    struct {
      BIT kfe$v_protect:1;	/*   Known file was installed protected */
      BIT kfe$v_lim:1;		/*   Linkable image */
      BIT kfe$v_procpriv:1;	/*   Use process privilege mask */
      BIT kfe$v_open:1;		/*   Image installed /OPEN */
      BIT kfe$v_hdrres:1;	/*   Image header block is resident */
      BIT kfe$v_shared:1;	/*   Image is shared */
      BIT kfe$v_shmident:1;	/*   Shared memory ident already set */
      BIT kfe$v_compatmod:1;	/*   Image is compatability mode */
      BIT kfe$v_nopurge:1;	/*   Image entry may not be purged */
      BIT kfe$v_account:1;	/*   Image level accounting */
      BIT kfe$v_writeable:1;	/*   Global sections are writeable */
      BIT kfe$v_exeonly:1;	/*   Image has only execute access allowed */
    } kfe$r_flags_bits;
  } kfe$r_flags_overlay;
  WORD kfe$w_gblseccnt;	/*   Global section count if shared */
  LONG kfe$l_usecnt;	/*   Usage counter */
  union {
    struct {
      LONG kfe$l_wcb;          /*   WCB address if open */
      LONG kfe$l_imghdr;       /*   Image header address if resident */
    } kfe$r_window_fields;
    WORD kfe$w_fid;		/*   File id */
    struct {
      WORD kfe$w_fid_num;	/*   File number field of file id */
      WORD kfe$w_fid_seq;	/*   File sequence number field of file id */
      WORD kfe$w_fid_rvn;	/*   Relative volume number field of file id */
    } kfe$r_fid_overlay;
  } kfe$r_window_overlay;
  LONG kfe$q_procpriv[2];
  BYTE kfe$b_matchctl;		/*   Global section match control */
  BYTE kfe$$fill_1;
  WORD kfe$w_amecod;		/*   Image header code specifying AME */
  LONG kfe$l_ident;		/*   Global section ident value */
  LONG kfe$l_orb;		/*   Address of Object Rights Block */
  WORD kfe$w_shrcnt;		/*   High water mark for sharing */
  BYTE kfe$b_filnamlen;		/*   Length of file name */
  BYTE kfe$t_filnam;		/*   Offset to file name */
}; 

/* *** MODULE $KFPBDEF ***
 *  
 *   KNOWN FILE POINTER BLOCK DEFINITIONS
 */
#define KFPB$K_LENGTH 16	/*   Length of pointer block */
#define KFPB$C_LENGTH 16	/*   Length of pointer block */
#define KFPB$S_KFPBDEF 16
struct kfpbdef {
  LONG kfpb$l_kfdlst;	/*   Device, Directory, Extension (KFD) list link */
  LONG kfpb$l_kfehshtab;	/*   Address of Known file name hash table */
  WORD kfpb$w_size;		/*   Size of pointer block in bytes */
  BYTE kfpb$b_type;		/*   Pointer block type */
  BYTE kfpb$b_spare;		/*   spare byte */
  WORD kfpb$w_kfdlstcnt;	/*   Number of entries in KFD list */
  WORD kfpb$w_hshtablen;	/*   Length of Hash table */
};
 
/* *** MODULE $KFRHDEF ***
 *  
 *   KNOWN FILE RESIDENT IMAGE HEADER DEFINITIONS
 */
#define KFRH$K_LENGTH 12	/*   Length of overhead area */
#define KFRH$C_LENGTH 12	/*   Length of overhead area */
#define KFRH$S_KFRHDEF 12
struct kfrhdef {
  LONG kfrh$l_bufend;	/*   Address of end of known file header */
  WORD kfrh$w_alias;	/*   Use secondary name on activation */
  WORD kfrh$w_size;	/*   Size of dynamic structure */
  BYTE kfrh$b_type;	/*   Dynamic structure type */
  BYTE kfrh$b_hdrver;	/*   Image header version */
  BYTE kfrh$t_ihd;	/*   Offset to decoded Image Header */
  /*  
   *   THE REMAINDER OF THIS STRUCTURE CONTAINS THE IMAGE HEADER OF THE
   *   SPECIFIED KNOWN FILE.  THE LOCATION KFE$L_IMGHDR IN THE KNOWN FILE
   *   ENTRY POINTS KFRH$C_LENGTH INTO THIS STRUCTURE, I.E AT THE IMAGE HEADER
   *   ITSELF.
   */
};
