#include descrip
#include dvidef

#include "status.h"

$DESCRIPTOR(date,"!%D");
char datebuff[24];
struct dsc$descriptor time =
         {23,DSC$K_DTYPE_T,DSC$K_CLASS_S,datebuff} ;

struct info {
  variant_union {
    char *ptr;
    variant_struct {
      short int just;
      short int width;
    } dummy$struct;
  } dummy$union;
};

cvt$$format(out,in,just,width)
  char *out, *in;
  short int just, width;
{
  /* out and in may point to the same address, move things in such a way
     as to not destroy things! */
  int size, i, front, back;

  size = strlen(in);
  if ((width == 0) || (width == size)) {	/* copy whole thing */
    if (out != in) {
      for (i = 0; i <= size; i++)
        out[i] = in[i];
    };
  }
  else if (just < 0) {	/* left justified */
    if (size > width) {	/* need to truncate end of string */
      strncpy(out,in,width);
      out[width] = '\0';
    }
    else {	/* need to pad spaces at end */
      strcpy(out,in);           
      for (i = size; i < width; i++)
        out[i] = ' ';
      out[width] = '\0';
    }
  }
  else if (just == 0) {	/* center justified */
    if (size > width) { /* need to truncate both ends of string */
      front = (size - width) / 2;	/* chop this many chars off front */
      back = (size - width + 1) / 2;	/* chop this many chars off back */
      for (i = front; i < (width - back); i++)
        out[i-front] = in[i];
      out[width] = '\0';
    }
    else {	/* need to pad spaces at both ends */
      front = (width - size) / 2;	/* add this many chars to front */
      back = (width - size + 1) / 2;	/* add this many chars to back */
      for (i = 0; i < size; i++)
        out[i + front] = in[i];
      for (i = 0; i < front; i++)
        out[i] = ' ';
      for (i = 0; i < back; i++)
        out[i + front + size] = ' ';
      out[width] = '\0';
    }
  }
  else {	/* right justified */
    if (size > width) { /* need to truncate start of string */
      for (i = 0; i < width; i++)
        out[i] = in[i + (size - width)];
      out[width] = '\0';
    }
    else {	/* need to pad spaces at start */
      for (i = size; i >= 0; i--)
        out[i + (width - size)] = in[i];
      for (i = 0; i < (width - size); i++)
        out[i] = ' ';
    }
  }
}

cvt$access_mode(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  char *modes[] = {"Kernel","Executive","Supervisor","User"};
  int asts, i, comma = 0;

  asts = *(int *)list[item]->bufadr;
  for (i=0; i<4; i++)
    if ((asts >> i) & 1) {
      if (comma) *str++ = ',';
      cvt$$format(str,modes[i],fmt[item].just,fmt[item].width);
      str += strlen(str);
      comma = 1;
    }
  if (! comma)
    cvt$$format(str,"none",fmt[item].just,fmt[item].width);
}

cvt$boolean(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  int val = 0;

  memcpy(&val, list[item]->bufadr, list[item]->length);
  val = (val & 1);

  cvt$$format(str,val?"true":"false",fmt[item].just,fmt[item].width);
}

cvt$cputim(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  int val = 0;
  int cputim[2];
  static int factor = -100000, zero = 0;
  short int len;

  memcpy(&val, list[item]->bufadr, list[item]->length);
  lib$emul(&factor,&val,&zero,cputim);
  sys$fao(&date,&len,&time,cputim);
  datebuff[len] = '\0';
  for (--len; (len > 0) && (datebuff[len] == ' ');--len)
    datebuff[len] = '\0';
  for (len = 0; datebuff[len] == ' '; ++len);

  cvt$$format(str,val?&datebuff[len]:"none",fmt[item].just,fmt[item].width);
}

cvt$creprc_flags(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  char *creflg[] = { "SSRWAIT", "SSFEXCU", "PSWAPM", "NOACNT", "BATCH",
                     "HIBER", "NOUAF", "NETWRK", "DISAWS", "DETACH",
                     "INTER", "IMGDMP", "CLISPEC", "NOPASSWORD" };
  int flags, i, comma = 0;

  flags = *(int *)list[item]->bufadr;
  for (i=0; i<14; i++)
    if ((flags >> i) & 1) {
      if (comma) *str++ = ',';
      cvt$$format(str,creflg[i],fmt[item].just,fmt[item].width);
      str += strlen(str);
      comma = 1;
    }
  if (! comma)
    cvt$$format(str,"none",fmt[item].just,fmt[item].width);
}

cvt$elapsed_time(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  int logintim[2] = {0, 0};
  int now[2];
  short int len;

  memcpy(logintim, list[item]->bufadr, list[item]->length);
  sys$gettim(now);
  lib$subx(logintim,now,logintim);
  sys$fao(&date,&len,&time,logintim);
  datebuff[len] = '\0';
  for (--len; (len > 0) && (datebuff[len] == ' ');--len)
    datebuff[len] = '\0'; /* trim trailing blanks */
  for (len = 0; datebuff[len] == ' '; ++len); /* trim leading blanks */

  cvt$$format(str,&datebuff[len],fmt[item].just,fmt[item].width);
}

cvt$epid_to_ipid(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  int val;

  val = epid_to_ipid(list[item]->bufadr);
  sprintf(str,"%08X",val);

  cvt$$format(str,str,fmt[item].just,fmt[item].width);
}

cvt$excvec(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  cvt$numeric_hex(item, list, fmt, str);
}

cvt$finalexc(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  cvt$numeric_hex(item, list, fmt, str);
}

cvt$hex_6(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  unsigned short int val[3] = {0,0,0};

  memcpy(val, list[item]->bufadr, list[item]->length);
  sprintf(str,"%04X%04X%04X", val[0], val[1], val[2]);
  cvt$$format(str,str,fmt[item].just,fmt[item].width);
}

cvt$hex_8(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  unsigned int val[2] = {0,0};

  memcpy(val, list[item]->bufadr, list[item]->length);
  sprintf(str,"%08X%08X", val[0], val[1]);
  cvt$$format(str,str,fmt[item].just,fmt[item].width);
}

cvt$hex_12(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  unsigned int val[3] = {0,0,0};

  memcpy(val, list[item]->bufadr, list[item]->length);
  sprintf(str,"%08X%08X%08X", val[0], val[1], val[2]);
  cvt$$format(str,str,fmt[item].just,fmt[item].width);
}

cvt$image(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  char *ptr;
  int max;

  /* figure out length of string */
  ptr = list[item]->bufadr + list[item]->length;
  /* find end of string */
  while ((ptr-- >= list[item]->bufadr) && ((*ptr == ' ') || (*ptr == '\0')));

  /* find .EXE; part of string */
  while ((ptr-- >= list[item]->bufadr) && (*ptr != '.'));
  *ptr = '\0';

  /* work backwards from fini to last ']' */
  while ((ptr-- >= list[item]->bufadr) && (*ptr != ']'));
  ++ptr; /* skip past the ']' */

  cvt$$format(str,ptr,fmt[item].just,fmt[item].width);
}

cvt$jobtype(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  unsigned char *ptr;
  static char *jobtype[] = { "Detached", "Network", "Batch",
                             "Local", "Dialup", "Remote" };

  ptr = list[item]->bufadr;
  cvt$$format(str,jobtype[*ptr],fmt[item].just,fmt[item].width);
}

cvt$type(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  int jobtyp;
  int mode;
  int owner;
  static char *jobtype[] = { "Detached", "Network", "Batch",
                             "Local", "Modem", "Remote" };

  jobtyp = *(unsigned char *)list[item]->bufadr;
  mode = *(unsigned char *)list[item + 1]->bufadr;
  owner = *(int *)list[item + 2]->bufadr;
  if ((jobtyp == 0) && (mode == 1))
    jobtyp = 1;

  cvt$$format(str,owner?"Subprocess":jobtype[jobtyp],
              fmt[item].just,fmt[item].width);
}

cvt$mode(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  unsigned char *ptr;
  static char *mode[] = { "Other", "Remote", "Batch", "Interactive" };

  ptr = list[item]->bufadr;
  cvt$$format(str,mode[*ptr],fmt[item].just,fmt[item].width);
}

cvt$msgmsk(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  char *msg[] = { "TEXT", "IDENT", "SEVERITY", "FACILITY" };
  int flags, i, comma = 0;

  flags = *(int *)list[item]->bufadr;
  for (i=0; i<4; i++)
    if ((flags >> i) & 1) {
      if (comma) *str++ = ',';
      cvt$$format(str,msg[i],fmt[item].just,fmt[item].width);
      str += strlen(str);
      comma = 1;
    }
  if (! comma)
    cvt$$format(str,"none",fmt[item].just,fmt[item].width);
}

cvt$numeric(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  int val = 0;

  memcpy(&val, list[item]->bufadr, list[item]->length);
  sprintf(str,"%d",val);
  cvt$$format(str,str,fmt[item].just,fmt[item].width);
}

cvt$numeric_hex(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  int val = 0;

  memcpy(&val, list[item]->bufadr, list[item]->length);
  sprintf(str,"%0*X", list[item]->length * 2, val);
  cvt$$format(str,str,fmt[item].just,fmt[item].width);
}

cvt$numeric_oct(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  int val = 0;

  memcpy(&val, list[item]->bufadr, list[item]->length);
  sprintf(str,"%0*o",list[item]->length * 8 / 3, val);
  cvt$$format(str,str,fmt[item].just,fmt[item].width);
}

cvt$pages(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  int ppgcnt, gpgcnt;

  memcpy(&ppgcnt, list[item]->bufadr, list[item]->length);
  memcpy(&gpgcnt, list[item+1]->bufadr, list[item+1]->length);
  sprintf(str,"%d",ppgcnt + gpgcnt);
  cvt$$format(str,str,fmt[item].just,fmt[item].width);
}

cvt$phdflags(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  char *phdflg[] = { "PFMFLG", "DALCSTX", "WSPEAKCHK", "NOACCVIO",
                     "IWSPEAKCK", "IMGDMP", "NO_WS_CHNG" };
  int flg, i, comma = 0;

  flg = *(int *)list[item]->bufadr;
  for (i=0; i < 7; i++)
    if ((flg >> i) & 1) {
      if (comma) *str++ = ',';
      cvt$$format(str,phdflg[i],fmt[item].just,fmt[item].width);
      str += strlen(str);
      comma = 1;
    }
  if (! comma)
    cvt$$format(str,"none",fmt[item].just,fmt[item].width);
}

cvt$physical_terminal(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  char buff[64];
  char node[10];
  int status,len, i;
  struct itmlst dvi[2];
  struct dsc$descriptor nodename = {7,DSC$K_DTYPE_T,DSC$K_CLASS_S,&node[1]} ;
  struct dsc$descriptor devname = {0,DSC$K_DTYPE_T,DSC$K_CLASS_S,0} ;
  struct TermInfo {
    long int duetim;
    long int pid;
    short int rtt_link;
  } terminfo;
  int nodenumber;
  char *ptr;

  dvi[0].length = sizeof(buff);
  dvi[0].code = DVI$_TT_PHYDEVNAM;
  dvi[0].bufadr = buff;
  dvi[0].retlen = &len;
  dvi[1].code = 0;
  dvi[1].length = 0;

  devname.dsc$w_length = list[item]->length;
  devname.dsc$a_pointer = list[item]->bufadr;
  status = term_info(&devname,&terminfo);	/* get rtt_link if remote */
  if ((status & 1) && terminfo.rtt_link) {
    for (i = 0; i < 10; ++i)
      node[i] = '\0';
    /* here we need to call decnetinfo passing it the link number */
    status = Get_Decnet_node(&terminfo.rtt_link,&nodenumber,&nodename);
    node[0] = '(';
    for(i = 7; (i > 0) && (node[i] == ' '); --i) /* trim off trailing spaces */
      node[i] = '\0';
    node[i+1] = ')';
  } else
    node[0] = '\0';

  /* figure out length of string */
  len = list[item]->length;
  ptr = list[item]->bufadr + list[item]->length;
  while ((--ptr >= list[item]->bufadr) && ((*ptr == ' ') || (*ptr == '\0')))
    --len;

  ptr = list[item]->bufadr;
  if (*ptr == '_') {
    ++ptr;
    --len;
  };

  for (i=0; i < len; i++) {
    str[i] = *ptr;
    ++ptr;
  };
  str[len] = '\0';

  if (str[len - 1] == ':')
    str[--len] = '\0';
    
  if (sys$getdviw(0,0,&devname,&dvi[0],0,0,0,0) & 1) {
    if (len) {
      buff[len] = '\0';
      if (buff[len - 1] == ':')
        buff[--len] = '\0';
      if (buff[0] == '_')
        sprintf(str,"%s%s",&buff[1],node);
      else
        sprintf(str,"%s%s",buff,node);
    }
    else
      sprintf(str,"(*)");
  }
  cvt$$format(str,str,fmt[item].just,fmt[item].width);
}

cvt$privilege(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  char *privs[] = {"CMKRNL", "CMEXEC", "SYSNAM", "GRPNAM", "ALLSPOOL",
                   "DETACH", "DIAGNOSE", "LOG_IO", "GROUP", "ACNT",
                   "PRMCEB", "PRMMBX", "PSWAPM", "ALTPRI", "SETPRV",
                   "TMPMBX", "WORLD", "MOUNT", "OPER", "EXQUOTA",
                   "NETMBX", "VOLPRO", "PHY_IO", "BUGCHK", "PRMGBL",
                   "SYSGBL", "PFNMAP", "SHMEM", "SYSPRV", "BYPASS",
                   "SYSLCK", "SHARE", "UPGRADE", /* <- 32 not used */
                   "DOWNGRADE", /* <- 33 not used */ "GRPPRV", "READALL",
                   "", "", /* <- 36 & 37 not used */ "SECURITY" };
  int priv0, priv1, i, comma = 0;

  priv0 = *(int *)list[item]->bufadr;
  for (i=0; i<32; i++)
    if ((priv0 >> i) & 1) {
      if (comma) *str++ = ',';
      cvt$$format(str,privs[i],fmt[item].just,fmt[item].width);
      str += strlen(str);
      comma = 1;
    }

  priv1 = *(int *)(list[item]->bufadr + 4);
  for (i=34; i<36; i++)
    if ((priv1 >> (i-32)) & 1) {
      if (comma) *str++ = ',';
      cvt$$format(str,privs[i],fmt[item].just,fmt[item].width);
      str += strlen(str);
      comma = 1;
    }

  if (! comma)
    cvt$$format(str,"none",fmt[item].just,fmt[item].width);
}

cvt$state(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  unsigned char *ptr;
  static char *state[] = { "", "COLPG","MWAIT","CEF","PFW","LEF","LEFO","HIB",
                           "HIBO","SUSP","SUSPO","FPG","COM","COMO","CUR" };

  ptr = list[item]->bufadr;
  cvt$$format(str,state[*ptr],fmt[item].just,fmt[item].width);
}

cvt$sts(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  char *sts[] = { "RES", "DELPEN", "FORCPEN", "INQUAN", "PSWAPM", "RESPEN",
                  "SSFEXC", "SSFEXCE", "SSFEXCS", "SSFEXCU", "SSRWAIT",
                  "SUSPEN", "WAKEPEN", "WALL", "BATCH", "NOACNT", "SWPVBN",
                  "ASTPEN", "PHDRES", "HIBER", "LOGIN", "NETWRK", "PWRAST",
                  "NODELET", "DISAWS", "INTER", "RECOVER", "SECAUDIT" };
  int flag, i, comma = 0;

  flag = *(int *)list[item]->bufadr;
  for (i=0; i<27; i++)
    if ((flag >> i) & 1) {
      if (comma) *str++ = ',';
      cvt$$format(str,sts[i],fmt[item].just,fmt[item].width);
      str += strlen(str);
      comma = 1;
    }
  if (! comma)
    cvt$$format(str,"none",fmt[item].just,fmt[item].width);
}

cvt$text(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  char *ptr;
  int max;

  /* figure out length of string */
  ptr = list[item]->bufadr + list[item]->length;
  while ((ptr-- >= list[item]->bufadr) && ((*ptr == ' ') || (*ptr == '\0')));
  *(++ptr) = '\0';

  cvt$$format(str,list[item]->bufadr,fmt[item].just,fmt[item].width);
}

cvt$time(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  int bintim[2] = {0, 0};
  short int len;

  memcpy(bintim, list[item]->bufadr, list[item]->length);
  sys$fao(&date,&len,&time,bintim);
  datebuff[len] = '\0';
  for (--len; (len > 0) && (datebuff[len] == ' ');--len)
    datebuff[len] = '\0';
  for (len = 0; datebuff[len] == ' '; ++len);

  cvt$$format(str,&datebuff[len],fmt[item].just,fmt[item].width);
}

cvt$uaf_flags(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  char *uaf[] = { "DISCTLY", "DEFCLI", "LOCKPWD", "CAPTIVE", "DISACNT",
                  "DISWELCOM", "DISMAIL", "NOMAIL", "GENPWD", "PWD_EXPIRED",
                  "PWD2_EXPIRED", "AUDIT", "DISREPORT", "DISRECONNECT",
                  "AUTOLOGIN" };
  int flag, i, comma = 0;

  flag = *(int *)list[item]->bufadr;
  for (i=0; i<4; i++)
    if ((flag >> i) & 1) {
      if (comma) *str++ = ',';
      cvt$$format(str,uaf[i],fmt[item].just,fmt[item].width);
      str += strlen(str);
      comma = 1;
    }
  if (! comma)
    cvt$$format(str,"none",fmt[item].just,fmt[item].width);
}

cvt$uic(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  $DESCRIPTOR(uicfmt, "!%I");
  short int len;

  sys$fao(&uicfmt,&len,str,*(int *)list[item]->bufadr);
  str[len] = '\0';
  cvt$$format(str,str,fmt[item].just,fmt[item].width);
}

cvt$where(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  char buff[64];
  int len, i;
  struct itmlst dvi[2];
  struct dsc$descriptor devname = {0,0,0,0};
  char *ptr;

  dvi[0].length = sizeof(buff);
  dvi[0].code = DVI$_TT_PHYDEVNAM;
  dvi[0].bufadr = buff;
  dvi[0].retlen = &len;
  dvi[1].code = 0;
  dvi[1].length = 0;

  /* figure out length of string */
  len = list[item]->length;
  ptr = list[item]->bufadr + list[item]->length;
  while ((--ptr >= list[item]->bufadr) && ((*ptr == ' ') || (*ptr == '\0')))
    --len;

  ptr = list[item]->bufadr;
  if (*ptr == '_') {
    ++ptr;
    --len;
  };

  for (i=0; i < len; i++) {
    str[i] = *ptr;
    ++ptr;
  };
  str[len] = '\0';

  if (str[len - 1] == ':')
    str[--len] = '\0';
    
  devname.dsc$w_length = len;
  devname.dsc$a_pointer = list[item]->bufadr;
  if (sys$getdviw(0,0,&devname,&dvi[0],0,0,0,0) & 1) {
    if (len) {
      buff[len] = '\0';
      if (buff[len - 1] == ':')
        buff[--len] = '\0';
      if (buff[0] == '_')
        cvt$$where(str,&buff[1],fmt[item].just,fmt[item].width);
      else
        cvt$$where(str,buff,fmt[item].just,fmt[item].width);
    }
    else {
      cvt$$format(str,"(*)",fmt[item].just,fmt[item].width);
    };
  }
  else
    cvt$$where(str,str,fmt[item].just,fmt[item].width);
}

cvt$$where(out,in,just,width)
  char out[], in[];
  short int just, width;
{
  char server[16], port[16];

  /* see if it's a LT type terminal, if not, then just format it */
  if ((in[0] == 'L') && (in[1] == 'T'))
    if (readport(in ,server ,port) & 1) {
      strcpy(in, server);
      strcat(in, "/");
      strcat(in, port);
    }

  /* now, here would be a good place to do a database lookup
     to return a real live location, based on terminal name, or
     server/port name. Should also add smarts to figure out remote
     terminals. */
  lookup_terminal(in,out,132); /* slightly bogus - should know max. length */

  cvt$$format(out,out,just,width);
}

cvt$who(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  cvt$$format(str,"not implemented",fmt[item].just,fmt[item].width);
}

cvt$literal(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  strcpy(str,fmt[item].ptr);
}

cvt$global_word(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  sprintf(str,"%d",*(short int *)list[item]);
  cvt$$format(str,str,fmt[item].just,fmt[item].width);
}

cvt$current_time(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  int now[2];
  short int len;

  sys$gettim(now);
  sys$fao(&date,&len,&time,now);
  datebuff[len] = '\0';
  for (--len; (len > 0) && (datebuff[len] == ' ');--len)
    datebuff[len] = '\0'; /* trim trailing blanks */
  for (len = 0; datebuff[len] == ' '; ++len); /* trim leading blanks */

  cvt$$format(str,&datebuff[len],fmt[item].just,fmt[item].width);
}

cvt$devices(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  static struct DevInfo {
    long int pid;
    short int len;
    char name[64];
  } devinfo[20];
  static char buff[64*20+63]; /* room for device names, plus commas */
  static int devinfosize = sizeof(struct DevInfo);
  static int class = -1;
  static int devinfocnt = 20;
  static int *pid;
  static int status,i,indx;

  indx = 0;
  pid = list[item]->bufadr;

  /* find all devices owned by this PID */
  status = devices(&class, pid, &devinfocnt, &devinfosize, devinfo);
  if (status & 1) {
    /* okay, move all the device names into the tmp buffer */
    i = 0;
    while (devinfo[i].len) {
      /* strip the leading underscore and trailing colon while moving */
      indx += sprintf(&buff[indx],"%.*s,",devinfo[i].len-2,&devinfo[i].name[1]);
      devinfo[i].len = 0;
      ++i;
    };
    if (indx) --indx;
  };
  buff[indx] = '\0';
  cvt$$format(str,buff,fmt[item].just,fmt[item].width);
}

cvt$idletime(item, list, fmt, str)
  int item;
  struct itmlst *list[];
  struct info fmt[];
  char *str;
{
  struct dsc$descriptor device = {0,DSC$K_DTYPE_T,DSC$K_CLASS_S,0} ;
  struct TermInfo {
    long int duetim;
    long int pid;
    short int rtt_link;
  } terminfo;
  char buff[10];
  int hrs,min,sec;
  int status;

  device.dsc$w_length = list[item]->length;
  device.dsc$a_pointer = list[item]->bufadr;

  status = term_info(&device,&terminfo);
  if ((status & 1) && (terminfo.duetim > 0)) {
    min = terminfo.duetim / 60;
    hrs = min / 60;
    sec = terminfo.duetim - (min * 60);
    min = min - (hrs * 60);
    sprintf(buff,"%d:%02d.%02d", hrs, min, sec);
  } else {
    if (status & 1)
      sprintf(buff,"0:00.00");
    else
      buff[0] = '\0';
  };
  cvt$$format(str,buff,fmt[item].just,fmt[item].width);
}
