#include iodef
#include descrip
#include dvidef
#include msgdef
#include nfbdef
#include ssdef
#include "starlet.h"
#include "server.h"

#define NETNAME "_NET:"
#define MBXNAME "STATUS-SERVER-MBX"

int io_complete();
int connection_handler();
int get_idle();

main()
{
  long int i,status, pendinglink;

  for (i = 0; i < MAXLINKS; ++i)
    gbl_links[i].buffer = gbl_buffers[i];
  allocate_jpi_space();

  init_server();
  /* okay, we just loop forever */
  while(1) {
    sleep(60);
    sys$canwak(0,0); /* in case there are any pending WAKEs left... */
    /* hey, we woke up, better be a good reason */
    pendinglink = 0;
    for (i = 0; i <= gbl_net.maxlinks; i++) {
      if (gbl_links[i].flags.pending) {
        pendinglink = -1;
        gbl_links[i].flags.pending = 0;
        if (parse(i)) {
          status = sys$qio(0,gbl_links[i].chan,IO$_READVBLK,
			&gbl_links[i].iosb,
			&io_complete,i,
			gbl_links[i].buffer,BUFFERSIZE,0,0,0,0);
        } /* end if (parse(i)) */
      } /* end if (gbl_links[i].flags.pending) */
    } /* end for () */
    /* okay, if we woke up for no reason, let's collect idle times.. */
    if (!pendinglink)
    {
      status = sys$dclast(&get_idle,0,0);
    }
  } /* end while (1) */
}

init_server()
{
  $DESCRIPTOR(mbxname,MBXNAME);
  $DESCRIPTOR(netname,NETNAME);
  $DESCRIPTOR(task,"STATUS");
  struct {
    unsigned char type;
    int end;
  } nfb_block = {NFB$C_DECLNAME,0};
  struct dsc$descriptor nfb = {sizeof(nfb_block),0,0,&nfb_block};
  long int status;

  /* initialize mailbox, assign a channel to NET, and associate the two */    
  status = sys$crembx(0,&gbl_net.mbx,0,0,0,0,&mbxname);
  if (!(status & 1)) return(status);
  status = sys$assign(&netname,&gbl_net.chan,0,&mbxname);
  if (!(status & 1)) return(status);

  /* declare the network task */
  status = sys$qiow(0,gbl_net.chan,IO$_ACPCONTROL,&gbl_net.iosb,
		0,0,&nfb,&task,0,0,0,0);
  if (!(status & 1)) return(status);
  if (!(gbl_net.iosb.status & 1)) return(gbl_net.iosb.status);

  /* set the ball rolling.. */
  status = sys$qio(0,gbl_net.mbx,IO$_READVBLK,&gbl_net.iosb,
		&connection_handler,0,
		&gbl_net.msg_union.buffer,sizeof(gbl_net.msg_union.buffer),
		0,0,0,0);
  return(status);
}

connection_handler()
{
  $DESCRIPTOR(netname,NETNAME);
  $DESCRIPTOR(mbxname,MBXNAME);
  struct dsc$descriptor ncb = {0,0,0,0};
  long int i,found,status, level;

  if (!(gbl_net.iosb.status & 1)) { /* something fatal has occured? */
    sys$exit(gbl_net.iosb.status);
  };
                         
  switch (gbl_net.msg_union.msg_struct.msgtype) {
    case MSG$_CONFIRM:
    case MSG$_INTMSG:
    case MSG$_REJECT:
#ifdef DEBUG
printf("In connection handler - CONFIRM/INTMSG/REJECT\n");
#endif
      /* I don't know if I really need to handle these in any way */
      /* I assume I can safely ignore them and continue happily along */
      break;

    case MSG$_ABORT:
    case MSG$_DISCON:
    case MSG$_EXIT:
    case MSG$_PATHLOST:
    case MSG$_PROTOCOL:
    case MSG$_THIRDPARTY:
    case MSG$_TIMEOUT:
      /* These all pretty much mean the same thing, disconnect this link */

      /* find the link that needs to get shut down */
      for (i = 0;
           (i <= gbl_net.maxlinks)
	   && (gbl_links[i].unit != gbl_net.msg_union.msg_struct.unit);
	   ++i);
      status = sys$dassgn(gbl_links[i].chan);
      if (!(status & 1)) sys$exit(status); /* fatal problem */

#ifdef DEBUG
printf("In connection handler ABORT-TIMEOUT, link %d\n",i);
#endif
      /* be sure to clear these suckers out.. */
      --gbl_net.activelinks;
      gbl_links[i].chan = 0;
      gbl_links[i].unit = 0;

      /* check to see if we've lowered the highest current link */
      for (;gbl_links[gbl_net.maxlinks].flags.connected==0;
           --gbl_net.maxlinks);
      break;

    case MSG$_NETSHUT:
      /* the network is shutting down. I don't think I really have to do
         any cleanup here, so I may as well just exit the program entirely */  
      sys$exit(1);
      break;

    case MSG$_CONNECT:
#ifdef DEBUG
printf("In connection handler CONNECT\n");
#endif
      ncb.dsc$w_length = gbl_net.msg_union.buffer[5 + 
				gbl_net.msg_union.buffer[4]];
      ncb.dsc$a_pointer = &gbl_net.msg_union.buffer[6 + 
			   	gbl_net.msg_union.buffer[4]];

      level = security_level(ncb.dsc$a_pointer);

      if ((level == 0) || (gbl_net.activelinks >= MAXLINKS)) {
        /* either they aren't cleared to do this, or ... */
        /* we don't have any more room for any links, so reject it */
        status = sys$qiow(0,gbl_net.chan,IO$_ACCESS | IO$M_ABORT,
			&gbl_net.iosb,0,0,0,&ncb,0,0,0,0);
        if (!(status & 1)) sys$exit(status); /* fatal problem */
        if (!(gbl_net.iosb.status & 1)) sys$exit(gbl_net.iosb.status);
        break; /* just fall out of this mess */
      }

      /* find empty slot */
      found = 0;
      i = 0;  
      while ((found == 0) && (i < MAXLINKS)){
        for (; (i < MAXLINKS) && gbl_links[i].chan; ++i);
        if (gbl_links[i].flags.connected)
          ++i;
        else
          found = -1;
      };
      if (i == MAXLINKS) { /* no links really available yet */
        /* we don't have any more room for any links, so reject it */
        status = sys$qiow(0,gbl_net.chan,IO$_ACCESS | IO$M_ABORT,
			&gbl_net.iosb,0,0,0,&ncb,0,0,0,0);
        if (!(status & 1)) sys$exit(status); /* fatal problem */
        if (!(gbl_net.iosb.status & 1)) sys$exit(gbl_net.iosb.status);
        break; /* just fall out of this mess */
      };

      /* okay, an empty slot was found */
      status = sys$assign(&netname,&gbl_links[i].chan,0,&mbxname);
      if (!(status & 1)) sys$exit(status); /* fatal problem ? */

      gbl_links[i].unit = get_unit(gbl_links[i].chan);
      if (gbl_links[i].unit == 0) { /* huh? fatal flaw */
        printf("Unable to get unit for channel %d\n",gbl_links[i].chan);
        sys$exit(4);
      };

#ifdef DEBUG
printf("-making connection, link %d\n",i);
#endif
      /* make the connection */
      status = sys$qiow(0,gbl_links[i].chan,IO$_ACCESS,&gbl_net.iosb,
			0,0,0,&ncb,0,0,0,0);
      if (!(status & 1)) sys$exit(status); /* fatal problem ? */
      if (!(gbl_net.iosb.status & 1)) sys$exit(gbl_net.iosb.status);

      /* save off the info */
      gbl_links[i].clearance = level;
      gbl_links[i].flags.connected = 1;
      gbl_links[i].flags.pending = 0;
      if (i > gbl_net.maxlinks)
        gbl_net.maxlinks = i;
      ++gbl_net.activelinks;

      /* get the ball rolling for this link */
      status = sys$qio(0,gbl_links[i].chan,IO$_READVBLK,&gbl_links[i].iosb,
			&io_complete,i,
			gbl_links[i].buffer,BUFFERSIZE,0,0,0,0);
      if (!(status & 1)) sys$exit(status); /* fatal problem ? */
      break;

    default:
      printf("Unknown connection request %d\n",
	gbl_net.msg_union.msg_struct.msgtype);
      sys$exit(4); /* fatal flaw I guess? */
      break;
  } /* end of switch */

  status = sys$qio(0,gbl_net.mbx,IO$_READVBLK,&gbl_net.iosb,
		&connection_handler,0,
		&gbl_net.msg_union.buffer,sizeof(gbl_net.msg_union.buffer),
		0,0,0,0);
  if (!(status & 1)) sys$exit(status);
}

io_complete(indx)
  int indx;
{
  if (!(gbl_links[indx].iosb.status & 1)) { /* problem completing read? */
    *gbl_links[indx].buffer = CLEANUP;   /* let someone know to clean up */
  };
  gbl_links[indx].flags.pending = 1;
  gbl_links[indx].iosb.status = 0;
  sys$wake(0,0);
};

get_unit(chan)
  unsigned short int chan;
{
  unsigned short int unit;
  struct itmlst unit_list[] = {{4,DVI$_UNIT,&unit,0},{0,0,0,0}};
  long int status;

  status = sys$getdviw(0,chan,0,unit_list,0,0,0,0);
  if (!(status & 1)) unit = 0;

  return (unit);
};

allocate_jpi_space()
{
  struct itmlst syilst[] = {{4,SYI$_MAXPROCESSCNT,&system_jpi.maxprocs,0}
			   ,{0,0,0,0}};
  int status,numbytes;

  status = sys$getsyiw(0,0,0,syilst,0,0,0);
  if (!(status & 1)) sys$exit(status);

  numbytes = (JPIMAXBUFFLEN + (4 * JPIMAXITEMCNT)) * system_jpi.maxprocs;

  if ((system_jpi.data = malloc(numbytes)) == 0) {
    printf("Unable to allocate memory for %d processes\n",
	system_jpi.maxprocs);
    sys$exit(4); /* die, should use an out of memory error */
  };

  status = add_jpi_item(JPI$_PROC_INDEX);
  status = add_jpi_item(JPI$_PID);
  status = add_jpi_item(JPI$_STS); /* used to see if PHDRES, or SUSPENDED */
  status = add_jpi_item(JPI$_MASTER_PID); /* used to zap NULL & SWAPPER */
  status = add_jpi_item(JPI$_CPUTIM); /* used to get idle times */
}
