/*
* ++
* FACILITY:
*
*	TW/PY driver test program
*
* ABSTRACT:
*		Program to take data from a users terminal input it to a PY
*	device and display output.  The program contains the ability to also
*	report interesting events on a control connection.
*
*	NOTE:
*		The program requires the following privileges
*
*			1) SHARE
*			2) PHY_IO
*			3) SYSPRV
*
* 1) The program will ask for a terminal to output diagnostic information.  If
*    you just hit <cr> it will skip outputying this info.
*
* 2) The diagnostic information is notification of XON & XOFF, and device
*    characteristics changing on the TW device.  If you use this you have
*    to have PHY_IO or else the program bombs.
*
* 3) The program will stop accepting input if the TW device is not accepting
*    input.  When this happens all you can do is wait, I did this on purpose
*    to make sure that the XOFF logic is working.  Some day maybe I will go
*    back and make it smarter.
* --
*/

/* Include file definitions and template structure definitions */

#include	descrip
#include	iodef
#include	ssdef
#include	stdio
#include	ttdef
#include	tt2def

struct	devchar
{
char		class;
char		type;
short int	buffer_size;
int		basic_char;
int		extended_char;
};

struct	iosb
{
short int	status;
short int	byte_cnt;
short int 	terminator;
short int	terminator_size;
};



/* Global data definitions */

char		mbx_buff[40];
char  		py_r_buff[255];

short int	info_chan;
short int	kbd_chan;
short int	mbx_chan;
short int	py_chan;

short int	kbd_s_efn;
short int	kbd_r_efn;
short int	py_s_efn;
short int	py_w_efn;
short int	wait_efn;
short int	xoff_efn;

int		exit_handler();
int		exit_status;
int		info_flag = 0;
int		term_mask[8] = {0, 0, 0, 0, 0, 0, 0, 0};

struct	devchar saved_char;
struct	exit_block
{
int		flink;
int		*hanlder_addr;
char		arg_count;
char		unused[3];
int		reason;
}		exit_block_descrip  = {0, exit_handler, 1, 0, 0, 0, &exit_status};
struct	iosb	info_w_iosb;
struct	iosb	kbd_w_iosb;
struct	iosb	mbx_iosb;
struct	iosb	py_r_iosb;
struct	iosb	py_s_iosb;
struct	term_descrip
{
short int	size;
short int	unused;
int		*ptr;
} term_block = {32, 0, &term_mask[0]};


main()
{

char		devnam[80];
char		kbd_r_buff[255];


$DESCRIPTOR(dummy,"");
$DESCRIPTOR(kbd_devnam,"tt:");
$DESCRIPTOR(null_char,"\000");
$DESCRIPTOR(py_devnam,"PYA0:");

int		bintim[2] = {-5*10*100*100,-1};
int		length;
int		read_mbx_ast();
int		py_read_ast();
int		set_line_ast();
int		xon_ast();
int		xoff_ast();
int		status;

struct	devchar	changed_char;
struct	iosb	kbd_r_iosb;
struct	iosb	py_w_iosb;


/* Get information channel name and assign a channel to it   */

printf("Enter device to output diagnostic information on: ");
gets(devnam);

if (length=strlen(devnam))
{
   dummy.dsc$a_pointer = &devnam[0];
   dummy.dsc$w_length = length;
   status = SYS$ASSIGN(&dummy, &info_chan, 0, 0);
   if (status & SS$_NORMAL)
   {
      info_flag = 1;
   }
   else
   {
      printf("Unable to assign infomation channel error number %9X", status);
   }
}


/* Create a mailbox, get a channel to the PY device and the terminal exit any of these steps fail */

status = SYS$CREMBX(0, &mbx_chan, 0, 0, 0, 0, 0);
if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);

status = SYS$ASSIGN(&py_devnam, &py_chan, 0, 0);
if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);

status = SYS$ASSIGN(&kbd_devnam, &kbd_chan, 0, 0);
if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);

/* Get a bunch of efn that we will need exit if cannot get them */
status = LIB$GET_EF(&kbd_s_efn);
if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
status = LIB$GET_EF(&kbd_r_efn);
if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
status = LIB$GET_EF(&py_s_efn);
if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
status = LIB$GET_EF(&py_w_efn);
if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
status = LIB$GET_EF(&wait_efn);
if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
status = LIB$GET_EF(&xoff_efn);
if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);


/* Now read the terminal characteristics so we can alter them */

status = SYS$QIOW(kbd_s_efn, kbd_chan, IO$_SENSEMODE, &kbd_r_iosb, 0, 0, &saved_char, 12, 0, 0, 0, 0);

if (status & SS$_NORMAL)
{
  if (kbd_r_iosb.status & SS$_NORMAL)
  {
    changed_char = saved_char;
    changed_char.basic_char = TT$M_NOECHO | changed_char.basic_char & ~(TT$M_HOSTSYNC | TT$M_TTSYNC);
    changed_char.extended_char = TT2$M_PASTHRU | changed_char.extended_char;
  }
  else
  {
    LIB$SIGNAL(kbd_r_iosb.status);
  }
}
else
{
  LIB$SIGNAL(status);
}


/* Now establish the exit handler and change the terminal characteristics */

status = SYS$DCLEXH (&exit_block_descrip);
if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);

status = SYS$QIOW(kbd_s_efn, kbd_chan, IO$_SETMODE, &kbd_r_iosb, 0, 0, &changed_char, 12, 0, 0, 0, 0);
if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
if (!(kbd_r_iosb.status & SS$_NORMAL)) LIB$SIGNAL(kbd_r_iosb.status);


/* All set up now so start read to mailbox, and PYDEVICE */

status = SYS$QIO (0, mbx_chan, IO$_READVBLK, &mbx_iosb, &read_mbx_ast, 0, &mbx_buff[0],
		  sizeof(mbx_buff), 0, 0, 0, 0);

status = SYS$QIO (0, py_chan, IO$_READVBLK, &py_r_iosb, &py_read_ast, 0, &py_r_buff[0],
		  sizeof(py_r_buff), 0, 0, 0, 0);


/* Set up XON, XOFF, and set line ASTs */

status = SYS$QIOW(py_s_efn, py_chan, IO$_SETMODE, &py_s_iosb, 0, 0, xon_ast, 0, 0, 1, 0, 0);
status = SYS$QIOW(py_s_efn, py_chan, IO$_SETMODE, &py_s_iosb, 0, 0, xoff_ast, 0, 0, 2, 0, 0);
status = SYS$QIOW(py_s_efn, py_chan, IO$_SETMODE, &py_s_iosb, 0, 0, set_line_ast, 0, 0, 3, 0, 0);


/* Now the real magic begins we get a read going to the terminal in an infinite loop */


printf("\n\nTo exit use CTRL-@\n\n");
dummy.dsc$a_pointer = &kbd_r_buff[0];

while (1)
{

  status = SYS$QIOW (kbd_r_efn, kbd_chan, IO$_READVBLK, &kbd_r_iosb, 0, 0, &kbd_r_buff[0], 1,
		     0, &term_block, 0, 0);
  if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);

  /* Stall for 50 milli-seconds waiting for more data */

  status = SYS$SETIMR (wait_efn, &bintim, 0, 0);
  if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
  status = SYS$WAITFR(wait_efn);
  if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);


  /* Now pick up any more user data */

  status = SYS$QIOW (kbd_r_efn, kbd_chan, IO$_READVBLK | IO$M_TIMED, &kbd_r_iosb, 0, 0, &kbd_r_buff[1],
		     (sizeof(kbd_r_buff)-1), 0, &term_block, 0, 0);
  if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
  dummy.dsc$w_length = 1 + kbd_r_iosb.byte_cnt;


  /* Go ahead and try and input a character */


  length = dummy.dsc$w_length;
  do
  {
    status = SYS$QIOW(py_w_efn, py_chan, IO$_WRITEVBLK, &py_w_iosb, 0, 0,
		      &kbd_r_buff[0], length, 0, 0, 0, 0);
    if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
    if ((py_w_iosb.status == SS$_DATAOVERUN) && (py_w_iosb.byte_cnt < length))
    {
      if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
      status = SYS$WAITFR(xoff_efn);
      if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
      status = SYS$SETAST(0);
      if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
      status = SYS$CLREF(xoff_efn);
      if (!(status & SS$_NORMAL)) LIB$SIGNAL(status);
      status = SYS$SETAST(1);
      length = length - py_w_iosb.byte_cnt;
    }
    else if ((py_w_iosb.status == SS$_NORMAL) || (py_w_iosb.status == SS$_DATAOVERUN))
    {
      break;
    }
    else
    {
      LIB$SIGNAL(py_w_iosb.status);
    }

  } while (py_w_iosb.status != SS$_NORMAL);

  status = LIB$MATCHC(&null_char, &dummy);
  if (status) LIB$SIGNAL(SS$_HANGUP);
}

}

/* Simple exit handler routine to reset terminal characteristics */


exit_handler()
{
int		status;

struct	iosb	exit_iosb;


status = SYS$QIOW(kbd_s_efn, kbd_chan, IO$_SETMODE, &exit_iosb, 0, 0, &saved_char, 12,
		  0, 0, 0, 0);
if (!(status & SS$_NORMAL)) printf("Error resetting terminal reason is %9X");

}

/* This routine will read the termination mailbox and report it to user */
read_mbx_ast()
{
int		status;


/* Indicate if read mailbox & if successful */
if (mbx_iosb.status & SS$_NORMAL)
{
  printf("\n\nTermination mailbox message received form TW device!!\n\n");
}
else
{
  printf("\n\nTermination maibox read with error %9X\n\n", mbx_iosb.status);
}


/* Reset read to termination mailbox */

status = SYS$QIOW(0, mbx_chan, IO$_READVBLK, &mbx_iosb, &read_mbx_ast, 0, &mbx_buff[0],
		  sizeof(mbx_buff), 0, 0, 0, 0);
if (!(status & SS$_NORMAL)) printf("\n\nError setting up MBX read reason is %9X\n\n");

}



/* This routine will write to the terminal data read from the PY device */
py_read_ast()
{
int		kbd_write_ast();
int		status;

if (!(py_r_iosb.status & SS$_NORMAL))
{
  printf("\n\nError reading from PY device program exiting!!!\n\n");
  LIB$SIGNAL(py_r_iosb.status);
}

status = SYS$QIO(0, kbd_chan, IO$_WRITEVBLK, &kbd_w_iosb, &kbd_write_ast, 0, &py_r_buff[0], py_r_iosb.byte_cnt, 0, 0, 0, 0);

if (!(status & SS$_NORMAL))
{
  printf("\n\nError writing PY data to terminal program exiting!!!\n\n");
  LIB$SIGNAL(status);
}

}



/* This rouinte gets called when the write to the terminal finishes. It restarts the read on the PYDEVICE. */
kbd_write_ast()
{
int		status;

status = SYS$QIO (0, py_chan, IO$_READVBLK, &py_r_iosb, &py_read_ast, 0, &py_r_buff[0], sizeof(py_r_buff), 0, 0, 0, 0);

if (!(status & SS$_NORMAL))
{
  printf("\n\nError reading from PY program exiting!!!\n\n");
  LIB$SIGNAL(status);
}

}

/* This routine will signal when an XOFF AST is delivered it resets AST and reports character that is delivered */
xoff_ast(param)
int		param;
{
char		buffer[40];

short int	signal_char;

int		len;
int		status;


status = SYS$QIOW (py_s_efn, py_chan, IO$_SETMODE, &py_s_iosb, 0, 0, &xoff_ast, 0, 0, 2, 0, 0);
if (!(status & SS$_NORMAL))
{
  printf("Error enabling XOFF ast on PYDEVICE");
  LIB$SIGNAL(status);
}
if (!(py_s_iosb.status & SS$_NORMAL))
{
  printf("Error enabling XOFF ast on PYDEVICE");
  LIB$SIGNAL(py_s_iosb.status);
}

if (info_flag)
{
  signal_char = (short int) (param >> 16);
  len =  sprintf(buffer, "\015\012Received XOFF character: %2X\015\012", signal_char);
  status = SYS$QIO(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);
}

}

/* This routine will signal when an XON AST is delivered it resets AST and reports character that is delivered */
xon_ast(param)
int		param;
{
char		buffer[40];

short int	signal_char;

int		len;
int		status;

status = SYS$SETEF(xoff_efn);

status = SYS$QIOW (py_s_efn, py_chan, IO$_SETMODE, &py_s_iosb, 0, 0, &xon_ast, 0, 0, 1, 0, 0);
if (!(status & SS$_NORMAL))
{
  printf("Error enabling XON ast on PYDEVICE");
  LIB$SIGNAL(status);
}
if (!(py_s_iosb.status & SS$_NORMAL))
{
  printf("Error enabling XON ast on PYDEVICE");
  LIB$SIGNAL(py_s_iosb.status);
}

if (info_flag)
{
  signal_char = (short int) (param >> 16);
  len =  sprintf(buffer, "\015\012Received XON character: %2X\015\012", signal_char);
  status = SYS$QIO(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);
}

}

/* This routine will signal when an XON AST is delivered it resets AST */
set_line_ast()
{
char		buffer[80];

int		len;
int		status;

struct	devchar	current_char;
struct	devchar	perm_char;

status = SYS$QIOW (py_s_efn, py_chan, IO$_SETMODE, &py_s_iosb, 0, 0, &set_line_ast, 0, 0, 3, 0, 0);
if (!(status & SS$_NORMAL))
{
  printf("Error enabling set line ast on PYDEVICE");
  LIB$SIGNAL(status);
}
if (!(py_s_iosb.status & SS$_NORMAL))
{
  printf("Error enabling set line ast on PYDEVICE");
  LIB$SIGNAL(py_s_iosb.status);
}

if (info_flag);
{
  status = SYS$QIOW(py_s_efn, py_chan, IO$_SENSEMODE, &py_s_iosb, 0, 0, &current_char, 12, 0, 0, 0, 0);
  if (!(status & SS$_NORMAL))
  {
    printf("Error reading current characteristics on PYDEVICE");
    LIB$SIGNAL(status);
  }
  if (!(py_s_iosb.status & SS$_NORMAL))
  {
    printf("Error reading current characteristics on PYDEVICE");
    LIB$SIGNAL(py_s_iosb.status);
  }

  len = sprintf(buffer,"\015\012 Current device characteristics \015\012");
  status = SYS$QIOW(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);
  len = sprintf(buffer,"Device class:        %8X\015\012",current_char.class);
  status = SYS$QIOW(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);
  len = sprintf(buffer,"Devic type:          %8X\015\012",current_char.type);
  status = SYS$QIOW(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);
  len = sprintf(buffer,"Buffer size:         %8X\015\012",current_char.buffer_size);
  status = SYS$QIOW(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);
  len = sprintf(buffer,"Basic chars & witdh:%8X\015\012",current_char.basic_char);
  status = SYS$QIOW(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);
  len = sprintf(buffer,"Extended chars.:     %8X\015\012",current_char.extended_char);
  status = SYS$QIOW(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);

  status = SYS$QIOW(py_s_efn, py_chan, IO$_SENSECHAR, &py_s_iosb, 0, 0, &perm_char, 8, 0, 0, 0, 0);
  if (!(status & SS$_NORMAL))
  {
    printf("Error reading current characteristics on PYDEVICE");
    LIB$SIGNAL(status);
  }
  if (!(py_s_iosb.status & SS$_NORMAL))
  {
    printf("Error reading current characteristics on PYDEVICE");
    LIB$SIGNAL(py_s_iosb.status);
  }

  len = sprintf(buffer,"\015\012Permanent device characteristics \015\012");
  status = SYS$QIOW(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);
  len = sprintf(buffer,"Device class:        %8X\015\012",perm_char.class);
  status = SYS$QIOW(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);
  len = sprintf(buffer,"Devic type:          %8X\015\012",perm_char.type);
  status = SYS$QIOW(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);
  len = sprintf(buffer,"Buffer size:         %8X\015\012",perm_char.buffer_size);
  status = SYS$QIOW(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);
  len = sprintf(buffer,"Basic chars & witdh: %8X\015\012",perm_char.basic_char);
  status = SYS$QIOW(0, info_chan, IO$_WRITEVBLK, &info_w_iosb, 0, 0, &buffer[0], len, 0, 0, 0, 0);

}

}
