	PROGRAM DISPLAY
C
C Author: Chris Doran, Sira Ltd.,
C		       South Hill, Chislehurst, Kent, BR7 5EH, England
C		       Tel: +44 1 467 2636, Telex 896649, Fax: +44 1 467 6515
C++
C Reads lines of text from SYS$INPUT, including VT200 special escape sequences,
C reformats them for VT100, VT52, or HP2648a terminals, and outputs to
C SYS$OUTPUT. This program was designed for use by the Sira CAI package, and
C hence only guarantees to support facilities used therein:-
C
C	esc [ row ; col H	position cursor
C	esc [ n A		cursor up
C	esc [ n B		cursor down
C	esc [ n C		cursor right
C	esc [ n D		cursor left
C	esc [ n J		clear screen
C	esc [ n K		clear line
C	esc [ n X		erase characters (overwrite with spaces)
C	esc [ top ; bot r	set scrolling region
C	esc # 3			double-width/double-height, top half
C	esc # 4			double-width/double-height, bottom half
C	esc # 5			single-width (i.e. normal)
C	esc # 6			double-width
C	esc ( B			define G0 set as ASCII
C	esc ( 0			define G0 set as line-drawing set
C	esc ( 9			define G0 as special down-line-loaded VT200 set
C	esc ) B			define G1 set as ASCII
C	esc ) 0			define G1 set as line-drawing set
C	esc ) 9			define G1 as special down-line-loaded VT200 set
C	^N			switch to G1 character set
C	^O			switch to G0 character set
C	esc [ m			Special rendition off
C	esc [ 1 m		Bold on
C	esc [ 4 m		Underscore on
C	esc [ 5 m		Blink on
C	esc [ 7 m		Inverse on
C	esc [ 22 m		Bold off
C	esc [ 24 m		Underscore off
C	esc [ 25 m		Blink off
C	esc [ 27 m		Inverse off
C	esc 7			Save cursor position, rendition, char set
C	esc 8			Restore cursor position, rendition, char set
C	esc D			Cursor down 1 line
C	esc M			Cursor up 1 line
C	esc E			Cursor to next line, col 1
C
C Plus the following special sequences:
C	esc [ h ; m ; s; t |	delays output of (whole of) the current line by
C				  the given time (t is 1/100's sec)
C
C	esc @			takes the current line as a prompt, displays it
C				  at the terminal, waits for CR to be pressed,
C				  deletes the line, and continues.
C
C DISPLAY can also create symbols. The escape sequence:
C
C	esc?symbol?		generates a symbol To_symbol, assigning to it
C				  the escape sequence to move to the current
C				  character position (if known).
C	esc!symbol!text!	generates symbol Show_symbol, assigning to it
C				  the escape sequence necessary to emphasise text
C				  (which is written at the current position),
C				and also symbol NoShow_symbol, assigning to it
C				  the escape sequence to remove the emphasis.
C				For VT100, VT200, VT300, and HP terminals, the
C				  emphasis is done by inverse video. For VT52's,
C				  a block character is placed at either end of
C				  the text.
C
C Also:	Restore_Cursor		is set to the string required to restore the
C				  cursor position etc. whenever esc 7 is
C				  processed.
C
C Note that the cursor position must be known to DISPLAY for the above symbols
C to produce the right effect. Default is top LH corner of screen. Use an
C esc [ H sequence if elsewhere.
C
C Any line with ! in column 1 is ignored as comment, BUT !! in cols 1-2 treats
C it as an accepted line beginning with a single !. !'s elsewhere are considered
C as normal characters, except that within an esc !symbol!text! sequence, !! is
C required to get a single !.
C
C In-line symbol substitution is also performed, a la DCL, i.e. 'SYMB' gets replaced
C by the current value of symbol SYMB, before each line is processed. If a '
C is not followed by a valid symbol name, or currently-defined symbol, then no
C substitution takes place, without warning (to handle literal '). A double
C prime ('') is also replaced by a single one (e.g. when you want to show
C primes around text which is, or might be, an actual symbol.
C--
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE '($DVIDEF)'
	INCLUDE '($DCDEF)'
	INCLUDE '($TTDEF)'
	CHARACTER*255 INPLIN,OUTLIN
	CHARACTER*128 SHOW,NOSHOW,SYMBOL,POSIT*10
	CHARACTER*1 ESC,TOG1,TOG0,CSI
	LOGICAL LIB$GETDVI,LIB$GET_SYMBOL,LIB$GET_FOREIGN,LIB$GET_INPUT
	LOGICAL*1 RENDOFF/.TRUE./, PROMPT/.FALSE./
	BYTE SPACING(24)/24*1/,LINEATTR(24)/24*5/
	PARAMETER (MAXN=4, ASCII=-1, LINE=0, MATHS=9,
     1		   VT100=1, VT200=2, VT52=3, HP2648A=4,
     2		   BLINK=1, INVERSE=2, UNDERLINE=4, BRIGHT=8,
     3		   TT$_HP2648A = 246, TT$_HP2648A_T = 181,
     4		   TOG1=CHAR(14), TOG0=CHAR(15), ESC=CHAR(27),
     5		   CSI=CHAR(155))
C     4		   TOG1=CHAR(78), TOG0=CHAR(79), ESC=CHAR(36),
C     5		   CSI=CHAR(37))
	INTEGER*2 LINP
	INTEGER ROW/1/,COL/1/,RENDITION/0/,
     1		INPG0SET/ASCII/,INPG1SET/LINE/,INPCURSET/ASCII/,
     2		OUTG1SET/ASCII/,OUTCURSET/ASCII/,
     3		SAVE_COL/1/,SAVE_ROW/1/,SAVE_REND/0/,SAVE_CSET/ASCII/,
     4		SAVE_G1SET/ASCII/
	INTEGER N(MAXN)
	INTEGER BMASK(7)/BRIGHT,BRIGHT,0,UNDERLINE,BLINK,0,INVERSE/
	COMMON //TTYPE,ROW,COL,INPLIN,IPTR,OUTLIN,OPTR,INPCURSET,SPACING
	IF(.NOT.LIB$GETDVI(DVI$_DEVCLASS,,'SYS$OUTPUT',TTYPE)) GOTO 10
	IF(TTYPE.NE.DC$_TERM) GOTO 10
	IF(.NOT.LIB$GETDVI(DVI$_DEVTYPE,,'SYS$OUTPUT',TTYPE)) GOTO 10
	IF (TTYPE.EQ.TT$_VT100 .OR.
     1		(TTYPE.GE.TT$_VT101.AND.TTYPE.LE.TT$_VT132)) THEN
	  TTYPE=VT100
	ELSE IF(TTYPE.EQ.TT$_VT200_series .OR.
     1		TTYPE.EQ.TT$_VT300_series) THEN
	  TTYPE=VT200
	ELSE IF(TTYPE.EQ.TT$_VT52 .OR. TTYPE.EQ.TT$_VT55) THEN
	  TTYPE=VT52
	  OUTG1SET=LINE
	ELSE IF(TTYPE.EQ.TT$_FT1 .OR. TTYPE.EQ.TT$_HP2648A .OR.
     1		TTYPE.EQ.TT$_HP2648A_T) THEN
	  TTYPE=HP2648A
	  OUTG1SET=MATHS
	ELSE
 10	  STOP 'UNSUPPORTED TERMINAL TYPE'
	ENDIF
C Read command line if there is one, stripping possible leading and trailing "
	IF(.NOT.LIB$GET_FOREIGN(OUTLIN,,LINP)) GOTO 9000
	IF(LINP.EQ.0) GOTO 7100
	IF(OUTLIN(1:1).EQ.'"') THEN
		LINP=LINP-1
		IF (OUTLIN(LINP+1:LINP+1).EQ.'"') LINP=LINP-1
		OUTLIN=OUTLIN(2:LINP+1)
	ENDIF
C
C Got a line, Make symbol substitutions until no more symbols.
C
 100	IPTR=1		! IPTR -> INPLIN = position of next char to store
	OPTR=0		! OPTR -> OUTLIN = position from which to look for prime
	NPTR=1		! NPTR -> OUTLIN = position of next char to copy
 110	J=INDEX(OUTLIN(OPTR+1:LINP),'''')
	OPTR=OPTR+J
	IF(J.NE.0) THEN
 120	  J=INDEX(OUTLIN(OPTR+1:LINP),'''')
	  IF(J-1) 130,122,125
 122	  SYMBOL(1:1)=''''		! Change '' to '
	  LSYMB=1			! by faking symbol
	  J=OPTR+J
	  GOTO 127
 125	  J=OPTR+J
	  IF(LIB$GET_SYMBOL(OUTLIN(OPTR+1:J-1),SYMBOL,LSYMB)) THEN
 127	    INPLIN(IPTR:IPTR+OPTR-NPTR+LSYMB-1)=
     1			OUTLIN(NPTR:OPTR-1)//SYMBOL(:LSYMB)
	    IPTR=IPTR+OPTR-NPTR+LSYMB
	    NPTR=J+1
	    OPTR=J
	    GOTO 110
	  ELSE
	    OPTR=J
	    GOTO 120
	  ENDIF
	ELSE
 130	  INPLIN(IPTR:IPTR+LINP-NPTR+1)=OUTLIN(NPTR:LINP)
	  LINP=IPTR+LINP-NPTR
	ENDIF
C
C Now process line.
C
	IPTR=1				! Pointer to input line
	IF (INPLIN(1:1).EQ.'!') THEN
	  IF (INPLIN(2:2).EQ.'!') THEN
	    IPTR=IPTR+1			! !! -> !
	  ELSE
	    GOTO 7100			! ! in col 1 marks comment
	  ENDIF
	ENDIF
	OPTR=1				! Pointer to output line
	IF (TTYPE.EQ.HP2648A) THEN
	  OUTCURSET=ASCII
	  IF (RENDITION.NE.0) CALL RENDSEQ(OUTLIN,OPTR,RENDITION)
	ENDIF
C Do a basic check for a lines with no complications:
C If the line does not contain ESC, CSI, ^O, ^N, or LF and the current character
C set is ASCII, it can be written out just as it is.
C If it doesn't contain ESC or CSI, but does contain ^N, ^O or LF, or the current
C I/O sets are not ASCII, it must be copied character by character, with character set
C substitutions and line counting, but it is unnecessary to do any escape
C sequence processing or position counting (since a CR is done at the end).
	IF (STR$FIND_FIRST_IN_SET(INPLIN(:LINP),ESC//CSI).EQ.0) THEN
	  IF (INPCURSET.EQ.ASCII .AND. OUTCURSET.EQ.ASCII .AND.
     1		STR$FIND_FIRST_IN_SET(INPLIN(:LINP),
     2			TOG0//TOG1//CHAR(10)).EQ.0) THEN
	    CALL LIB$PUT_OUTPUT(INPLIN(:LINP))
	    GOTO 7000
	  ELSE
	    DO IPTR=1,LINP
	      IF(INPLIN(IPTR:IPTR).EQ.TOG0) THEN
		INPCURSET=INPG0SET
	      ELSE IF(INPLIN(IPTR:IPTR).EQ.TOG1) THEN
		INPCURSET=INPG1SET
	      ELSE
		CALL ADDCHR(OUTLIN,OPTR,OUTCURSET,OUTG1SET)
		IF (INPLIN(IPTR:IPTR).EQ.CHAR(10)) THEN
		  IF (ROW.EQ.24) THEN
		    DO I=1,23					! Scroll up
		      SPACING(I)=SPACING(I+1)
		      LINEATTR(I)=LINEATTR(I+1)
		    ENDDO
		    SPACING(24)=1
		    LINEATTR(24)=5
		  ELSE
		    ROW=ROW+1
		  ENDIF
		ENDIF
	      ENDIF
	    ENDDO
	  ENDIF
	ELSE
C Full character-by-character processing needed.
	  SSTART=0			! Pointer to start of Show_symbol sequence
	  SPACING(ROW)=1		! Character-character spacing
	  LINEATTR(ROW)=5		! Line attributes (esc # n) -- 5 = normal
	  DO WHILE (IPTR.LE.LINP)
	    IF(INPLIN(IPTR:IPTR).EQ.ESC .OR.
     1		INPLIN(IPTR:IPTR).EQ.CSI) THEN
	      EPTR=IPTR
C Escape sequence starting.
	      IPTR=IPTR+1
	      IF (INPLIN(IPTR-1:IPTR-1).EQ.CSI) GOTO 140	! csi = esc [
	      IF (INPLIN(IPTR:IPTR).EQ.'[') THEN
C esc [. Allow 1-4 integers to follow, with ';' separator.
		IPTR=IPTR+1
 140		DO I=1,MAXN
		  N(I)=0
		  DO WHILE (INPLIN(IPTR:IPTR).GE.'0' .AND.
     1			  INPLIN(IPTR:IPTR).LE.'9')
		    N(I)=N(I)*10+ICHAR(INPLIN(IPTR:IPTR))-ICHAR('0')
		    IPTR=IPTR+1
		  ENDDO
		  IF(INPLIN(IPTR:IPTR).EQ.';') IPTR=IPTR+1
		ENDDO
C Handle position: esc [ row ; col H
		IF (INPLIN(IPTR:IPTR).EQ.'H') THEN
		  ROW=MAX0(1,N(1))
		  COL=MAX0(1,N(2))
		  IF(TTYPE.LE.VT200) GOTO 5000
		  CALL POSSEQ(OUTLIN,OPTR)
C Handle clear rest of screen: esc [ n J
C	n=0	erase rest of screen
C	n=1	erase to cursor
C	n=2	erase whole screen
C The cursor is unchanged.
		ELSE IF (INPLIN(IPTR:IPTR).EQ.'J') THEN
		  IF (N(1).EQ.0) THEN
		    DO I=ROW+1,24
		      SPACING(I)=1
		      LINEATTR(I)=5
		    ENDDO
		    IF(TTYPE.LE.VT200) GOTO 5000
		    OUTLIN(OPTR:OPTR+6)=ESC//'J'//ESC//'*dhA'
		    OPTR=OPTR+2
		    IF (TTYPE.EQ.HP2648A .AND. ROW.EQ.1 .AND. COL.EQ.1)
     1			OPTR=OPTR+5
		  ELSE IF (N(1).EQ.1) THEN
		    DO I=1,ROW-1
		      SPACING(I)=1
		      LINEATTR(I)=5
		    ENDDO
		    IF(TTYPE.LE.VT200) GOTO 5000
C No erase to cursor. Overwrite with spaces instead.
		    OUTLIN(OPTR:OPTR+1)=ESC//'H'
		    OPTR=OPTR+2
		    IF(ROW.NE.1) THEN
		      DO I=1,ROW-1
			OUTLIN(OPTR:OPTR+3)=ESC//'K'//CHAR(10)//' '
			OPTR=OPTR+3
		      ENDDO
		    ENDIF
		    OUTLIN(OPTR:OPTR+COL)=' '		! Clear line inc cursor
		    OUTLIN(OPTR+COL:OPTR+COL)=CHAR(8)	! Reposition cursor
		    OPTR=OPTR+COL+1
		  ELSE		! assume to be 2. Clear all & reposition cursor
		    DO I=1,24
		      SPACING(I)=1
		      LINEATTR(I)=5
		    ENDDO
		    IF(TTYPE.LE.VT200) GOTO 5000
		    OUTLIN(OPTR:OPTR+3)=ESC//'H'//ESC//'J'//ESC//'*dhA'
		    OPTR=OPTR+4
		    IF (TTYPE.EQ.HP2648A) OPTR=OPTR+5 ! Clear graphics too
		    CALL POSSEQ(OUTLIN,OPTR)
		  ENDIF
C Handle clear rest of line: esc [ n K
C	n=0	erase rest of line
C	n=1	erase to cursor
C	n=2	erase whole line
C The cursor is unchanged.
		ELSE IF (INPLIN(IPTR:IPTR).EQ.'K') THEN
		  IF(TTYPE.LE.VT200) GOTO 5000
		  IF (N(1).EQ.0) THEN
		    OUTLIN(OPTR:OPTR+1)=ESC//'K'
		    OPTR=OPTR+2
		  ELSE IF (N(1).EQ.1) THEN
C No erase to cursor. Overwrite with spaces instead.
		    OUTLIN(OPTR:OPTR+COL*SPACING(ROW))=CHAR(13)
		    OPTR=OPTR+COL*SPACING(ROW)+2
		    OUTLIN(OPTR-1:OPTR-1)=CHAR(8)		! Reposition cursor
		  ELSE		! assume to be 2. Clear all & reposition cursor
		    OUTLIN(OPTR:OPTR+3)=CHAR(13)//ESC//'K '
		    OPTR=OPTR+3
		    IF (TTYPE.EQ.HP2648A) THEN
		      CALL HPRC(COL,'C',OUTLIN,OPTR)
		    ELSE
		      CALL POSSEQ(OUTLIN,OPTR)
		    ENDIF
		  ENDIF
C Handle enhancements:
C	0 = all off
C	1 = bright		22 = not bright
C	4 = underlined		24 = not underlined
C	5 = blink		25 = not blinking
C	7 = inverted		27 = not inverted
C Enhancements are ignored for VT52's.
		ELSE IF (INPLIN(IPTR:IPTR).EQ.'m') THEN
		  IF(TTYPE.NE.VT52) THEN
		    RENDOFF=.FALSE.
		    IF(N(1).EQ.0) THEN
		      RENDITION=0
		    ELSE
		      DO I=1,MAXN
			IF(N(I).GT.20) THEN
			  RENDITION=RENDITION.AND..NOT.BMASK(N(I)-20)
			  RENDOFF=.TRUE.
			ELSE IF(N(I).NE.0) THEN
			  RENDITION=RENDITION.OR.BMASK(N(I))
			ENDIF
		      ENDDO
		    ENDIF
C Handle any esc [ 2x m's for VT100.
		    IF(RENDOFF.AND.(TTYPE.EQ.VT100)) THEN
		      OUTLIN(OPTR:OPTR+2)=ESC//'[m'	    ! All off
		      OPTR=OPTR+3
		      IF (RENDITION.NE.0)
     1			CALL RENDSEQ(OUTLIN,OPTR,RENDITION) ! and turn back on
		      GOTO 6000
		    ELSE IF (TTYPE.LE.VT200) THEN
		      GOTO 5000
		    ENDIF
C Ignoring if VT52, so only thing left is HP2648a.
		    IF(COL.NE.80) CALL RENDSEQ(OUTLIN,OPTR,RENDITION)
		  ENDIF ! not VT52
C Reposition cursor in same line or column.
C	esc [ n A	up n lines
C	esc [ n B	down n lines
C	esc [ n C	right n columns
C	esc [ n D	left n columns
		ELSE IF (INPLIN(IPTR:IPTR).GE.'A' .AND.
     1			INPLIN(IPTR:IPTR).LE.'D') THEN
		  N(1)=MAX0(1,N(1))
		  IF (INPLIN(IPTR:IPTR).EQ.'A') THEN
		    ROW=MAX0(1,ROW-N(1))
		  ELSE IF (INPLIN(IPTR:IPTR).EQ.'B') THEN
		    ROW=MIN0(24,ROW+N(1))
		  ELSE IF (INPLIN(IPTR:IPTR).EQ.'C') THEN
		    COL=MIN0(80,COL+N(1))
		  ELSE ! 'D'
		    COL=MAX0(1,COL-N(1))
		  ENDIF	
		  IF (TTYPE.LE.VT200) GOTO 5000
		  CALL POSSEQ(OUTLIN,OPTR)
		ELSE IF (INPLIN(IPTR:IPTR).EQ.'r') THEN
C esc [ bot ; top r	Set scrolling region. This is partially simulated as follows:
C	VT52	set terminal to HOLD mode. The user must press SHIFT/SCROLL
C		  to continue when the screen fills. This is not a true emulation,
C		  as the top of the screen is lost.
C	HP2648A	set memory lock at line top. There is no definition of the
C		  bottom of the scroll region, which remains at 24+.
C If bot=1, then cancel hold mode (VT52) or memory lock (HP).
C In each case, the cursor moves to home (as it does on a real VT200).
C However, we do not take account of the following:-
C	On a VT200, with scroll set, esc [ n A and esc [ n B will not move outside
C	  the scrolling region, whilst esc [ l ; c H will.
C	On an HP2648a with memory lock set, esc A and esc B can move above the
C	  memory lock line, esc H moves to it, esc & a l R can only move within
C	  it, but esc & a l Y can move anywhere. Subsequent lines output will be
C	  counted as lines 24+..., even though they scroll off the top of the
C	  screen.
C The caller must handle these peculiarities. DISPLAY can't, because it doesn't
C really know the state of the terminal on entry (i.e. it may actually be in
C scroll mode at the start). However, it does always use screen-relative moves
C (esc & a l Y) on the HP.
		  OLDROW=ROW
		  ROW=1
		  COL=1
		  OUTLIN(OPTR:OPTR)=ESC
		  IF (TTYPE.EQ.VT52) THEN
		    OUTLIN(OPTR+1:OPTR+1)='['			! hold
		    IF (N(1).EQ.1) OUTLIN(OPTR+1:OPTR+1)='\'	! nohold
		  ELSE IF (TTYPE.EQ.HP2648A) THEN
		    IF (N(1).EQ.1) THEN
		      OUTLIN(OPTR+1:OPTR+1)='m'			! lock off
		    ELSE
		      IF (N(1).NE.OLDROW) CALL HPRC(N(1),'Y',OUTLIN,OPTR)
		      OUTLIN(OPTR:OPTR+2)=ESC//'l'		! lock on
		    ENDIF
		  ELSE
		    GOTO 5000				! Output as is for VT100/200
		  ENDIF
		  OPTR=OPTR+2				! Count previous esc seq
		  CALL POSSEQ(OUTLIN,OPTR)		! Set VT52/HP to home
C Erase n characters (overwrite with spaces): esc [ n X
		ELSE IF (INPLIN(IPTR:IPTR).EQ.'X') THEN
		  IF (TTYPE.EQ.VT200) GOTO 5000		! VT200 only
		  IF (TTYPE.NE.VT100) N(1)=N(1)*SPACING(ROW)
		  OUTLIN(OPTR:OPTR+N(1))=' '
		  OPTR=OPTR+N(1)
		  CALL POSSEQ(OUTLIN,OPTR)		! cursor doen't move
C Special delay function: esc [ h ; m ; s ; t |
		ELSE IF (INPLIN(IPTR:IPTR).EQ.'|') THEN
		  CALL LIB$WAIT(FLOAT(N(1)*3600+N(2)*60+N(3))+
     1			FLOAT(N(4))/100.)
		ELSE ! unsupported esc [. Just copy (but may be corrupted)
		  GOTO 5000
		ENDIF
C Not a supported esc [, try esc ( = define main (G0) character set. The
C character sets supported are '0', which is the standard VT100 set, and '9',
C which is a special down-lineloaded set for the VT220 emulating an HP terminal,
C or the HP terminal itself.
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.'(') THEN
		CALL CHANGE_SET(INPG0SET)
C esc ) is similar, but defines the alternate character set.
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.')') THEN
		CALL CHANGE_SET(INPG1SET)
C Deal with line attributes -- esc #3/4/5/6. Note that these should occur at
C the very beginning of a line for correct simulation. On a VT100/VT200, they
C can be passed straight through with no special handling.
	      ELSE IF(INPLIN(IPTR:IPTR).EQ.'#' .AND. TTYPE.GT.VT200) THEN
		IPTR=IPTR+1
		LINEATTR(ROW)=ICHAR(INPLIN(IPTR:IPTR))-ICHAR('0')
		IF (LINEATTR(ROW).NE.5) SPACING(ROW)=2	      
C Save cursor position, character attributes, alternate set, and current set.
C Also writes a symbol "Restore_cursor" which may be used externally. For a
C VT100/VT200, this is just esc 8, for other terminals, we output the actual
C sequence required.
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.'7') THEN
		SAVE_COL=COL
		SAVE_ROW=ROW
		SAVE_REND=RENDITION
		SAVE_CSET=INPCURSET
		SAVE_G1SET=INPG1SET
		IF (TTYPE.LE.VT200) THEN
		  SYMBOL(:2)=ESC//'8'
		  LSYMB=2
		ELSE
		  LSYMB=1
		  CALL POSSEQ(SYMBOL,LSYMB)
		  CALL RENDSEQ(SYMBOL,LSYMB,RENDITION)
		  IF (TTYPE.EQ.VT52) THEN
		    SYMBOL(LSYMB:LSYMB+1)=ESC//'G'
		    IF (OUTCURSET.EQ.LINE) SYMBOL(LSYMB+1:LSYMB+1)='F'
		    LSYMB=LSYMB+1
		  ELSE ! must be HP2648a
		    SYMBOL(LSYMB:LSYMB+3)=ESC//')A'//TOG0
		    IF (OUTG1SET.EQ.LINE) SYMBOL(LSYMB+2:LSYMB+2)='B'
		    IF (OUTCURSET.NE.ASCII) SYMBOL(LSYMB+3:LSYMB+3)=TOG1
		    LSYMB=LSYMB+3
		  ENDIF
		ENDIF
		CALL LIB$SET_SYMBOL('Restore_Cursor',SYMBOL(:LSYMB))
		IF (TTYPE.LE.VT200) GOTO 5000
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.'8') THEN
		COL=SAVE_COL
		ROW=SAVE_ROW
		RENDITION=SAVE_REND
		INPCURSET=SAVE_CSET
		INPG1SET=SAVE_G1SET
		IF (TTYPE.LE.VT200) GOTO 5000
		CALL POSSEQ(OUTLIN,OPTR)
		CALL RENDSEQ(OUTLIN,OPTR,RENDITION)
C
C esc D = move down one line, with a scroll up if at bottom line. Note that this
C is different to esc [ A, which doesn't scroll. The exact function is performed
C by LF on a VT52, but on an HP2648a, we must delete a line at the top of
C the screen if a scroll is required.
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.'D') THEN
		OLDROW=ROW
		ROW=MIN0(24,ROW+1)
		IF (TTYPE.LE.VT200) GOTO 5000
		IF (TTYPE.EQ.VT52 .OR. OLDROW.NE.24) THEN
		  OUTLIN(OPTR:OPTR+1)=CHAR(10)
		  OPTR=OPTR+1
		ELSE
		  IF (TTYPE.EQ.VT52) THEN
		    OUTLIN(OPTR:OPTR+1)=ESC//'I'
		    OPTR=OPTR+2
		  ELSE ! HP2648A
		    OUTLIN(OPTR:OPTR+5)=ESC//'B'//ESC//'M'//ESC//'A' ! up, del l, down
		    OPTR=OPTR+6
		    CALL HPRC(COL,'C',OUTLIN,OPTR) ! esc M sets column to 1
		  ENDIF
		ENDIF
C esc E is the same, but also does a carriage-return.
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.'E') THEN
		OLDROW=ROW
		ROW=MIN0(24,ROW+1)
		COL=1
		IF (TTYPE.LE.VT200) GOTO 5000
		IF (TTYPE.EQ.VT52 .OR. OLDROW.NE.24) THEN
		  OUTLIN(OPTR:OPTR+1)=CHAR(13)//CHAR(10)
		  OPTR=OPTR+2
		ELSE
		  IF (TTYPE.EQ.VT52) THEN
		    OUTLIN(OPTR:OPTR+3)=ESC//'I'//CHAR(13)//' '
		    OPTR=OPTR+3
		  ELSE ! HP2648A
		    OUTLIN(OPTR:OPTR+5)=ESC//'B'//ESC//'M'//ESC//'A' ! up, del l, down
		    OPTR=OPTR+6
		  ENDIF
		ENDIF
C esc M does cursor up, with scroll down if row=1. The same function is given
C by esc I on a VT52. On an HP2648a, we must insert a blank line at the top
C if row=1.
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.'M') THEN
		OLDROW=ROW
		IF (ROW.EQ.1) THEN
		  DO I=2,24
		    SPACING(I)=SPACING(I-1)
		    LINEATTR(I)=LINEATTR(I-1)
		  ENDDO
		  SPACING(1)=1
		  LINEATTR(1)=5
		ENDIF
		ROW=MAX0(1,ROW-1)
		IF (TTYPE.LE.VT200) GOTO 5000
		OUTLIN(OPTR:OPTR+1)=ESC//'A'
		OPTR=OPTR+2
		IF (TTYPE.EQ.VT52) THEN
		  OUTLIN(OPTR-1:OPTR-1)='I'
		ELSE IF(OLDROW.EQ.1) THEN		! must be HP2648A so don't test
		  OUTLIN(OPTR-1:OPTR+3)='M'//ESC//'B'//ESC//'L' ! del l 24, to 1, insert
		  OPTR=OPTR+4
		  CALL HPRC(COL,'C',OUTLIN,OPTR) ! esc L sets column to 1
		ENDIF
C esc @ defines this line as a prompt
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.'@') THEN
		PROMPT=.TRUE.
C Handle esc ? To_symbol ? definition.
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.'?') THEN
		J=LIB$LOCC('?',INPLIN(IPTR+1:LINP))+IPTR
		I=1
		CALL POSSEQ(POSIT,I)
		CALL LIB$SET_SYMBOL('To_'//INPLIN(IPTR+1:J-1),POSIT(:I-1))
		IPTR=J
C Handle Show_symbol definition. For the moment, we just store the start position
C and character set. The rest will be handled when the closing ! is found.
C For a VT52, we start 1 column back, with a filled rectangle. In order for the
C replacements to be made correctly, it is a restriction that the characters
C immediately preceding and following the shown string must be printing, and in
C the ASCII character set.
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.'!') THEN
		SSTART=LIB$LOCC('!',INPLIN(IPTR+1:LINP))+IPTR
		SYMBOL='Show_'//INPLIN(IPTR+1:SSTART-1)
		SPTR=1
		CALL POSSEQ(SHOW,SPTR)
		NOSHOW=SHOW(:SPTR-1)
		NSPTR=SPTR
		SHOCURSET=ASCII
		SHOALTSET=LINE
		NSCURSET=ASCII
		NSALTSET=LINE
		IF (TTYPE.LE.VT200) THEN
		  SHOW(SPTR:SPTR+3)=ESC//'[7m'
		  SPTR=SPTR+4
		  IF ((RENDITION.AND.INVERSE).EQ.0) THEN
		    IF (TTYPE.EQ.VT200) THEN
		      NOSHOW(NSPTR:NSPTR+4)=ESC//'[27m'
		      NSPTR=NSPTR+5
		    ELSE ! VT100 has no individual clears
		      NOSHOW(NSPTR:NSPTR+2)=ESC//'[m'
		      NSPTR=NSPTR+3
		      I=RENDITION.AND.(BLINK.OR.UNDERLINE.OR.BRIGHT)
		      IF (I.NE.0) CALL RENDSEQ(NOSHOW,NSPTR,I)
		    ENDIF
		  ENDIF
		ELSE IF (TTYPE.EQ.VT52) THEN
		  SHOW(4:4)=CHAR(ICHAR(SHOW(4:4))-1)
		  NOSHOW(4:4)=CHAR(ICHAR(NOSHOW(4:4))-1)
		  SHOW(SPTR:SPTR+2)=ESC//'Fa'
		  SPTR=SPTR+3
		  SHOCURSET=LINE
		  IPTR=IPTR-2
		  IF (INPLIN(IPTR:IPTR).LE.' ') THEN
		    NOSHOW(NSPTR:NSPTR)=' '
		    NSPTR=NSPTR+1
		  ELSE
		    CALL ADDCHR(NOSHOW,NSPTR,NSCURSET,NSALTSET)
		  ENDIF
		  IPTR=IPTR+2
		ELSE ! HP2648a
		  CALL RENDSEQ(SHOW,SPTR,RENDITION.OR.INVERSE)
		  IF ((RENDITION.AND.INVERSE).EQ.0)
     1			CALL RENDSEQ(NOSHOW,NSPTR,RENDITION)
		ENDIF
		IPTR=SSTART
		SSTART=SSTART+1
	      ELSE ! unsupported escape sequence, just copy (but may be corrupted)
		GOTO 5000
	      ENDIF
C Not escape, see if ! = end of symbol definition.
	    ELSE IF(INPLIN(IPTR:IPTR).EQ.'!' .AND. SSTART.GT.0) THEN
	      IF (INPLIN(IPTR+1:IPTR+1).EQ.'!') THEN
C !! in a Show_ sequence maps to !.
		IPTR=IPTR+1
		GOTO 500
	      ENDIF
	      IF (TTYPE.EQ.VT52) THEN
C For VT52, we just put a full rectangle at the other end (current position) of
C the display sequence for Show, and remove it for NoShow.
		CALL POSSEQ(SHOW,SPTR)
		NOSHOW(NSPTR:NSPTR+3)=SHOW(SPTR-4:SPTR-1)
		NSPTR=NSPTR+4
		SHOW(SPTR:SPTR+2)='a'//ESC//'G'
		SPTR=SPTR+3
		IPTR=IPTR+1
		IF (INPLIN(IPTR:IPTR).LE.' ') THEN
		  NOSHOW(NSPTR:NSPTR)=' '
		  NSPTR=NSPTR+1
		ELSE
		  CALL ADDCHR(NOSHOW,NSPTR,NSCURSET,NSALTSET)
		ENDIF
		IPTR=IPTR-1
C For VT100, VT200, and HP2648A, select correct character sets, and (for SHOW)
C turn rendition off. Must move to end point to do this for HP2648a, unless
C the end-point is column 80, where renditions go off automatically.
	      ELSE
		IF (TTYPE.EQ.HP2648A.AND.COL.LT.80)
     1			CALL HPRC(COL,'C',SHOW,SPTR)
		CALL STANDARD(SHOW,SPTR,INVERSE,SHOCURSET,SHOALTSET)
		CALL STANDARD(NOSHOW,NSPTR,0,NSCURSET,NSALTSET)
	      ENDIF
	      CALL LIB$SET_SYMBOL(SYMBOL,SHOW(:SPTR-1))
	      CALL LIB$SET_SYMBOL('No'//SYMBOL,NOSHOW(:NSPTR-1))
	      SSTART=0
C Deal with character set changes -- just note for the present.
	    ELSE IF(INPLIN(IPTR:IPTR).EQ.TOG1) THEN
	      INPCURSET=INPG1SET
	    ELSE IF(INPLIN(IPTR:IPTR).EQ.TOG0) THEN
	      INPCURSET=INPG0SET
C Anything else is just a normal character. Copy, allowing for character set
C changes.
	    ELSE
 500	      OLDSET=OUTCURSET
	      IF (LINEATTR(ROW).EQ.4.AND.INPLIN(IPTR:IPTR).GT.' ') THEN
		OUTLIN(OPTR:OPTR)='='		! Bottom of dh/dw line
		OPTR=OPTR+1
	      ELSE
		CALL ADDCHR(OUTLIN,OPTR,OUTCURSET,OUTG1SET)
	      ENDIF
	      IF (LINEATTR(ROW).NE.5.AND.INPLIN(IPTR:IPTR).GE.' ') THEN
		OUTLIN(OPTR:OPTR)=' '		! double-width -- double-space
		OPTR=OPTR+1
	      ENDIF
	      IF (SSTART.GT.0) THEN
		IF (TTYPE.LE.VT200) THEN
		  CALL ADDCHR(SHOW,SPTR,SHOCURSET,SHOALTSET)
		  CALL ADDCHR(NOSHOW,NSPTR,NSCURSET,NSALTSET)
		ELSE IF (TTYPE.EQ.HP2648A .AND. OUTCURSET.NE.OLDSET) THEN
C  A character set change on the HP2648a acts ends rendition, so reset/clear it.
		  I=SPTR
		  CALL HPRC(COL,'C',SHOW,SPTR)
		  NOSHOW(NSPTR:NSPTR+SPTR-I-1)=SHOW(I:SPTR-1)
		  NSPTR=NSPTR+SPTR-I
		  CALL RENDSEQ(SHOW,SPTR,RENDITION.OR.INVERSE)
		  CALL RENDSEQ(NOSHOW,NSPTR,RENDITION)
		ENDIF
	      ENDIF
	      IF (INPLIN(IPTR:IPTR).EQ.CHAR(8)) THEN
		COL=MAX0(1,COL-1)
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.CHAR(9)) THEN
		COL=MIN0(80,((COL+7).AND.(.NOT.7))+1)
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.CHAR(10)) THEN
		IF (ROW.EQ.24) THEN				! Scroll up
		  DO I=1,23		
		    SPACING(I)=SPACING(I+1)
		    LINEATTR(I)=LINEATTR(I+1)
		  ENDDO
		  SPACING(24)=1
		  LINEATTR(24)=5
		ELSE
		  ROW=ROW+1
		ENDIF
	      ELSE IF (INPLIN(IPTR:IPTR).EQ.CHAR(13)) THEN
		COL=1
	      ELSE
		COL=MIN0(80,COL+1)
	      ENDIF
	    ENDIF
	    GOTO 6000
C
C Copy escape sequence as it comes, for VT100/VT200/VT300, except that if
C this is a VT100, a starting csi must be changed to esc [.
 5000	    IF (TTYPE.EQ.VT100 .AND. INPLIN(EPTR:EPTR).EQ.CSI) THEN
	      OUTLIN(OPTR:OPTR+1) = ESC//'['
	      OUTLIN(OPTR+2:OPTR+IPTR-EPTR+1)=INPLIN(EPTR+1:IPTR)
	      OPTR=OPTR+1
	    ELSE
	      OUTLIN(OPTR:OPTR+IPTR-EPTR)=INPLIN(EPTR:IPTR)
	    ENDIF
	    OPTR=OPTR+IPTR-EPTR+1
C
 6000	    IPTR=IPTR+1
	  ENDDO
	ENDIF
C End of line.
	IF (PROMPT) THEN
	  CALL LIB$GET_COMMAND(INPLIN,OUTLIN(:OPTR-1))
	  PROMPT=.FALSE.
C Go back up 1 line, and erase prompt
	  IF (TTYPE.GT.VT200) THEN
	    OUTLIN=ESC//'A'//ESC//'K'
	    OPTR=5
	  ELSE
	    OUTLIN=ESC//'M'//ESC//'[K'
	    OPTR=6
	  ENDIF
	ENDIF
	CALL LIB$PUT_OUTPUT(OUTLIN(:OPTR-1))
	COL=1
 7000	ROW=ROW+1
 7100	IF(LIB$GET_INPUT(OUTLIN,,LINP)) GOTO 100
C
C All done. Leave output in standard state:
 9000	OPTR=1
	CALL STANDARD(OUTLIN,OPTR,RENDITION,OUTCURSET,OUTG1SET)
	IF(OPTR.NE.1) THEN
	  OUTLIN(OPTR:OPTR+1)=ESC//'['			! up 1 line
	  IF (TTYPE.LE.VT200) OPTR=OPTR+1		! Keep [ if ANSI
	  OUTLIN(OPTR+1:OPTR+1)='A'
	  CALL LIB$PUT_OUTPUT(OUTLIN(:OPTR+1))
	ENDIF
	CALL EXIT
	END
C
C Read esc ( <set> or esc ) <set>. Note change.
C
	SUBROUTINE CHANGE_SET(SET)
	IMPLICIT INTEGER (A-Z)
	CHARACTER*255 INPLIN,OUTLIN
	PARAMETER (ASCII=-1)
	BYTE SPACING(24)
	COMMON //TTYPE,ROW,COL,INPLIN,IPTR,OUTLIN,OPTR,INPCURSET,SPACING
	IPTR=IPTR+1
	I=ICHAR(INPLIN(IPTR:IPTR))-ICHAR('0')
	IF (I.EQ.ICHAR('B')-ICHAR('0')) I=ASCII		! 'B' is ASCII
	IF (INPCURSET.EQ.SET) INPCURSET=I
	SET=I
	RETURN
	END
C
C Place in STRING the escape sequence to position to ROW,COL, returning
C string length.
C
	SUBROUTINE POSSEQ(STRING,PTR)
	IMPLICIT INTEGER (A-Z)
	CHARACTER*(*) STRING,INPLIN*255,OUTLIN*255,ESC*1
	PARAMETER (VT100=1, VT200=2, VT52=3, HP2648A=4, ESC=CHAR(27))
C	PARAMETER (VT100=1, VT200=2, VT52=3, HP2648A=4, ESC=CHAR(36))
	BYTE SPACING(24)
	COMMON //TTYPE,ROW,COL,INPLIN,IPTR,OUTLIN,OPTR,INPCURSET,SPACING
	GOTO (100,200,52,2648), TTYPE
C VT100 and VT200
 100	CONTINUE
 200	STRING(PTR:PTR+1)=ESC//'['
	PTR=PTR+2
	IF(ROW.NE.1) CALL OUTINT(ROW,STRING,PTR)
	IF(COL.NE.1) THEN
	  STRING(PTR:PTR)=';'
	  PTR=PTR+1
	  CALL OUTINT(COL,STRING,PTR)
	ENDIF
	STRING(PTR:PTR)='H'
	PTR=PTR+1
	RETURN
C VT52
 52	IF(ROW.EQ.1 .AND. COL.EQ.1) THEN
	  STRING(PTR:PTR+1)=ESC//'H'
	  PTR=PTR+2
	ELSE
	  STRING(PTR:PTR+3)=ESC//'Y'//CHAR(31+ROW)//
     1					CHAR(32+(COL-1)*SPACING(ROW))
	  PTR=PTR+4
	ENDIF
	RETURN
C HP 2648A
 2648	CALL HPRC(COL,'c',STRING,PTR)			! esc & a <col> c
	CALL OUTINT(ROW-1,STRING,PTR)
	STRING(PTR:PTR)='Y'				! screen-relative row
	PTR=PTR+1
	RETURN
	END
C
C Write row or column only for HP. LETTR (R or C) dictates which.
C
	SUBROUTINE HPRC(I,LETTR,STRING,PTR)
	IMPLICIT INTEGER (A-Z)
	CHARACTER*(*) STRING,ESC*1,LETTR*1,INPLIN*255,OUTLIN*255
	BYTE SPACING(24)
	COMMON //TTYPE,ROW,COL,INPLIN,IPTR,OUTLIN,OPTR,INPCURSET,SPACING
	PARAMETER (ESC=CHAR(27))
C	PARAMETER (ESC=CHAR(36))
	STRING(PTR:PTR+3)=ESC//'&a '
	PTR=PTR+3
	II=I-1
	IF(LETTR.EQ.'C' .OR. LETTR.EQ.'c') II=II*SPACING(ROW)
	CALL OUTINT(II,STRING,PTR)
	STRING(PTR:PTR)=LETTR
	PTR=PTR+1
	RETURN
	END
C
C Write a 1-2 digit decimal number to STRING(PTR:), updating pointer.
C
	SUBROUTINE OUTINT(VAL,STRING,PTR)
	IMPLICIT INTEGER (A-Z)
	CHARACTER*(*) STRING
	IF (VAL.GT.9) THEN
	  STRING(PTR:PTR)=CHAR(VAL/10+ICHAR('0'))
	  PTR=PTR+1
	ENDIF
	STRING(PTR:PTR)=CHAR(MOD(VAL,10)+ICHAR('0'))
	PTR=PTR+1
	RETURN
	END
C
C Output character special rendition.
C
	SUBROUTINE RENDSEQ(STRING,PTR,RENDITION)
	IMPLICIT INTEGER (A-Z)
	PARAMETER (BLINK=1, INVERSE=2, UNDERLINE=4, BRIGHT=8,
     1		   VT100=1, VT200=2, VT52=3, HP2648A=4)
	CHARACTER*(*) STRING,ESC*1,INPLIN*255,OUTLIN*255
	PARAMETER (ESC=CHAR(27))
C	PARAMETER (ESC=CHAR(36))
	CHARACTER*1 CODE(0:3) /'5','7','4','1'/
	BYTE SPACING(24)
	COMMON //TTYPE,ROW,COL,INPLIN,IPTR,OUTLIN,OPTR,INPCURSET,SPACING
	IF (TTYPE.LE.VT200) THEN
	  STRING(PTR:PTR+2)=ESC//'['
	  PTR=PTR+3
	  IF (RENDITION.NE.0) THEN
	    DO I=0,3
	      IF (BTEST(RENDITION,I)) THEN
		STRING(PTR-1:PTR)=CODE(I)//';'
		PTR=PTR+2
	      ENDIF
	    ENDDO
	  ENDIF
	  STRING(PTR-1:PTR-1)='m'
C Don't put change sequence on HP terminal if already at column 80 -- rendition
C changes back automatically anyway, and if you do a change with the cursor at
C column 80, then the character there gets changed.
	ELSE IF (TTYPE.EQ.HP2648A .AND. COL.LT.80) THEN
	  I=RENDITION
C No bright, so change to underline.
	  IF((RENDITION.AND.BRIGHT).NE.0)
     1		 I=(I.AND..NOT.BRIGHT).OR.UNDERLINE
	  STRING(PTR:PTR+3)=ESC//'&d'//CHAR(I+ICHAR('@'))
	  PTR=PTR+4
	ENDIF ! no rendition on VT52
	RETURN
	END
C
C Create a string resetting to standard state:
C	rendition off
C	no alternate character set
	SUBROUTINE STANDARD(STRING,PTR,RENDITION,CURSET,ALTSET)
	IMPLICIT INTEGER (A-Z)
	CHARACTER*(*) STRING,INPLIN*255,OUTLIN*255,ESC*1
	PARAMETER (ASCII=-1, LINE=0, MATHS=9,
     1		   VT100=1, VT200=2, VT52=3, HP2648A=4,
     2		   ESC=CHAR(27))
C     2		   ESC=CHAR(36))
	BYTE SPACING(24)
	COMMON //TTYPE,ROW,COL,INPLIN,IPTR,OUTLIN,OPTR,INPCURSET,SPACING
	IF (RENDITION.NE.0) CALL RENDSEQ(STRING,PTR,0)
	IF (TTYPE.LE.VT200.AND.ALTSET.NE.LINE) THEN
	  STRING(PTR:PTR+2)=ESC//')0'
	  PTR=PTR+3
	  ALTSET=LINE
	ELSE IF (TTYPE.EQ.HP2648A.AND.ALTSET.NE.MATHS) THEN
	  STRING(PTR:PTR+2)=ESC//')A'
	  PTR=PTR+3
	  ALTSET=MATHS
	ENDIF
	IF (CURSET.NE.ASCII) THEN
	  IF(TTYPE.EQ.VT52) THEN
	    STRING(PTR:PTR+1)=ESC//'G'
	    PTR=PTR+2
C Don't put ^O on HP terminal if already at column 80 -- set changes back
C automatically anyway, and if you do have a ^O with the cursor at column 80,
C then the character there gets changed.
	  ELSE IF (TTYPE.NE.HP2648A .OR. COL.LT.80) THEN
	    STRING(PTR:PTR)=CHAR(15)
	    PTR=PTR+1
	  ENDIF
	ENDIF
	RETURN
	END
C
C Append character to output buffer. Make an appropriate substitution and/or
C character set change if necessary.
C
	SUBROUTINE ADDCHR(STRING,PTR,CURSET,ALTSET)
	IMPLICIT INTEGER (A-Z)
	CHARACTER*(*) STRING
	CHARACTER*255 INPLIN,OUTLIN
	CHARACTER*1 CHR,VT52CHR(95:126),HPCHR(95:126),HP9CHR(14),
     1		    VT9CHR(14),ALTCHR(4),ESC,HT,FF,NL,CR,VT,LF,TOG1,TOG0,
     2		    QRS52(ICHAR('q'):ICHAR('s'))
	PARAMETER (ASCII=-1, LINE=0, CTRL=8, MATHS=9,
     1		   VT100=1, VT200=2, VT52=3, HP2648A=4,
     2		   HT=CHAR(9), FF=CHAR(12), LF=CHAR(10),
     3		   CR=CHAR(13), VT=CHAR(11), NL=CHAR(0),
     4		   TOG1=CHAR(14), TOG0=CHAR(15), ESC=CHAR(27))
C     4		   TOG1=CHAR(78), TOG0=CHAR(79), ESC=CHAR(36))
	BYTE VT52SET(95:126),HPSET(95:126),SPACING(24)
	COMMON //TTYPE,ROW,COL,INPLIN,IPTR,OUTLIN,OPTR,INPCURSET,SPACING
	DATA	VT52CHR(95),VT52SET(95),HPCHR(95),HPSET(95)
     *			/' ',ASCII,' ',ASCII/		! space
     *		VT52CHR(96),VT52SET(96),HPCHR(96),HPSET(96)
     *			/'j',LINE,'(',MATHS/		! diamond -> -:-
     *		VT52CHR(97),VT52SET(97),HPCHR(97),HPSET(97)
     *			/'a',LINE,'c',LINE/		! solid square
     *		VT52CHR(98),VT52SET(98),HPCHR(98),HPSET(98)
     *			/'h',ASCII,HT,CTRL/		! HT
     *		VT52CHR(99),VT52SET(99),HPCHR(99),HPSET(99)
     *			/'f',ASCII,FF,CTRL/		! FF
     *		VT52CHR(100),VT52SET(100),HPCHR(100),HPSET(100)
     *			/'c',ASCII,CR,CTRL/		! CR
     *		VT52CHR(101),VT52SET(101),HPCHR(101),HPSET(101)
     *			/'l',ASCII,LF,CTRL/		! LF
     *		VT52CHR(102),VT52SET(102),HPCHR(102),HPSET(102)
     *			/'"',ASCII,'0',MATHS/		! degree
     *		VT52CHR(103),VT52SET(103),HPCHR(103),HPSET(103)
     *			/'g',LINE,'%',MATHS/		! +/-
     *		VT52CHR(104),VT52SET(104),HPCHR(104),HPSET(104)
     *			/'n',ASCII,NL,CTRL/		! NL
	DATA	VT52CHR(105),VT52SET(105),HPCHR(105),HPSET(105)
     *			/'v',ASCII,VT,CTRL/		! VT
     *		VT52CHR(106),VT52SET(106),HPCHR(106),HPSET(106)
     *			/'''',ASCII,'g',LINE/		! _|
     *		VT52CHR(107),VT52SET(107),HPCHR(107),HPSET(107)
     *			/'.',ASCII,'t',LINE/		! ~|
     *		VT52CHR(108),VT52SET(108),HPCHR(108),HPSET(108)
     *			/'.',ASCII,'r',LINE/		! |~
     *		VT52CHR(109),VT52SET(109),HPCHR(109),HPSET(109)
     *			/'`',ASCII,'f',LINE/		! |_
     *		VT52CHR(110),VT52SET(110),HPCHR(110),HPSET(110)
     *			/'+',ASCII,'/',LINE/		! -+-
     *		VT52CHR(111),VT52SET(111),HPCHR(111),HPSET(111)
     *			/'l',LINE,'~',ASCII/		! bar at scan 1
     *		VT52CHR(112),VT52SET(112),HPCHR(112),HPSET(112)
     *			/'n',LINE,'^',ASCII/		! bar at scan 3
     *		VT52CHR(113),VT52SET(113),HPCHR(113),HPSET(113)
     *			/'p',LINE,',',LINE/		! - (bar at scan 5)
	DATA	VT52CHR(114),VT52SET(114),HPCHR(114),HPSET(114)
     *			/'r',LINE,'_',ASCII/		! _ (bar at scan 7)
     *		VT52CHR(115),VT52SET(115),HPCHR(115),HPSET(115)
     *			/'_',ASCII,'_',ASCII/		! _ (bar at scan 9)
     *		VT52CHR(116),VT52SET(116),HPCHR(116),HPSET(116)
     *			/'|',ASCII,'5',LINE/		! |-
     *		VT52CHR(117),VT52SET(117),HPCHR(117),HPSET(117)
     *			/'|',ASCII,'6',LINE/		! -|
     *		VT52CHR(118),VT52SET(118),HPCHR(118),HPSET(118)
     *			/'^',ASCII,'8',LINE/		! _|_
     *		VT52CHR(119),VT52SET(119),HPCHR(119),HPSET(119)
     *			/'v',ASCII,'7',LINE/		! T
     *		VT52CHR(120),VT52SET(120),HPCHR(120),HPSET(120)
     *			/'|',ASCII,'.',LINE/		! |
     *		VT52CHR(121),VT52SET(121),HPCHR(121),HPSET(121)
     *			/'<',ASCII,'<',ASCII/		! <=
     *		VT52CHR(122),VT52SET(122),HPCHR(122),HPSET(122)
     *			/'>',ASCII,'>',ASCII/		! >=
	DATA	VT52CHR(123),VT52SET(123),HPCHR(123),HPSET(123)
     *			/'n',ASCII,'p',MATHS/		! pi
     *		VT52CHR(124),VT52SET(124),HPCHR(124),HPSET(124)
     *			/'#',ASCII,'>',LINE/		! =/=
     *		VT52CHR(125),VT52SET(125),HPCHR(125),HPSET(125)
     *			/'#',ASCII,'#',ASCII/		! pound
     *		VT52CHR(126),VT52SET(126),HPCHR(126),HPSET(126)
     *			/'.',ASCII,'.',ASCII/		! dec pt
C Characters used from "set 9":
	DATA HP9CHR/'_','\','^','[',';','1','2','$','#',':','s',
     1		    'w','q','a'/
	DATA VT9CHR/'k','h','<','^','=',9*'*'/
C Breakpoints for characters which may be in alternative sets:-
	DATA ALTCHR/'_','_','^','!'/
C Alternative line-drawing characters if VT52 currently in ASCII set
	DATA QRS52/'-','_','_'/
C
	CHR=INPLIN(IPTR:IPTR)
	SET=ASCII
	IF (INPCURSET.EQ.MATHS) THEN
	  I=LIB$LOCC(CHR,'V><^qtuvwxjklm')
	  IF (I.EQ.0) GOTO 1000
	  GOTO (100,200,300,400),TTYPE
	ENDIF
	IF (INPCURSET.NE.ASCII .AND. CHR.GT.'^')
     1		GOTO (110,210,310,410),TTYPE
	GOTO 1000
C
C "MATHS" sets.
C
C VT100.
 100	IF (I.EQ.1) THEN
	  CHR='v'
	ELSE
	  SET=LINE
	ENDIF
	GOTO 1000
C VT52.
 300	CHR=VT9CHR(I)
	IF (I.LE.2) SET=LINE
	GOTO 1000
C HP
 400	CHR=HP9CHR(I)
	SET=LINE
	IF (I.LE.4) SET=MATHS
	GOTO 1000
C
C Line-drawing sets
C
C VT52
 310	IF (CURSET.EQ.ASCII .AND. CHR.GE.'q' .AND. CHR.LE.'s') THEN
	  CHR=QRS52(ICHAR(CHR))	! use - (q) or _ (r or s)
	ELSE
	  SET=VT52SET(ICHAR(CHR))
	  CHR=VT52CHR(ICHAR(CHR))
	ENDIF
	GOTO 1000
C HP2648A.
 410	SET=HPSET(ICHAR(CHR))
	CHR=HPCHR(ICHAR(CHR))
	GOTO 1000
C VT100 requires no mapping for line-drawing set.
 110	CONTINUE
C VT200 requires no character mapping anywhere.
 210	CONTINUE
 200	SET=INPCURSET
C
C Change character set if required, in a terminal-dependent fashion.
C No set change is required for characters of or below:-
C	'^'	on VT100 and VT200
C	']'	on VT52
C	' '	on HP
C Note that the CTRL set only gets selected for HPs, so there is no need to
C check the terminal type. Further, it is used on a character-by-character
C basis only, otherwise enhancement sequences get displayed.
 1000	IF (SET.EQ.CTRL) THEN
	  STRING(PTR:PTR+6)=ESC//'Y'//CHR//ESC//'Z'//CHAR(8)//CHAR(8)
	  PTR=PTR+7
	ELSE
	  IF (SET.NE.CURSET .AND. CHR.GE.ALTCHR(TTYPE)) THEN
	    IF (SET.EQ.ASCII) THEN
	      IF (TTYPE.EQ.VT52) THEN
		STRING(PTR:PTR+1)=ESC//'G'
		PTR=PTR+2
	      ELSE
		STRING(PTR:PTR)=TOG0
		PTR=PTR+1
	      ENDIF
	    ELSE
	      IF (SET.NE.ALTSET.AND.TTYPE.NE.VT52) THEN
		STRING(PTR:PTR+1)=ESC//')'
		IF (TTYPE.LE.VT200) THEN
		  IF(SET.EQ.LINE) THEN
		    STRING(PTR+2:PTR+2)='0'
		  ELSE
		    STRING(PTR+2:PTR+2)='9'
		  ENDIF
		ELSE ! HP2848A
		  IF(SET.EQ.LINE) THEN
		    STRING(PTR+2:PTR+2)='B'
		  ELSE
		    STRING(PTR+2:PTR+2)='A'
		  ENDIF
		ENDIF
		PTR=PTR+3
		ALTSET=SET
	      ENDIF
	      IF (TTYPE.EQ.VT52) THEN
		STRING(PTR:PTR+1)=ESC//'F'
		PTR=PTR+2
	      ELSE
		STRING(PTR:PTR)=TOG1
		PTR=PTR+1
	      ENDIF
	    ENDIF
	    CURSET=SET
	  ENDIF
	  STRING(PTR:PTR)=CHR
	  PTR=PTR+1
	ENDIF
	RETURN
	END
